// $Id: TELL1FIRProcessEngine.cpp,v 1.2 2007/08/27 10:44:58 szumlat Exp $
// STL
#include<vector>
#include<algorithm>

// local
#include "TELL1Engine/TELL1FIRProcessEngine.h"

// std
#include <iostream>

//-----------------------------------------------------------------------------
// Implementation file for class : TELL1FIRProcessEngine
//
// 2007-08-10 : Tomasz Szumlak
//-----------------------------------------------------------------------------

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
TELL1FIRProcessEngine::TELL1FIRProcessEngine(  ):
  TELL1Engine(),
  m_K0 ( 1 ),
  m_K1 ( 0 ),
  m_K2 ( 0 )
{ }
//=============================================================================
// Destructor
//=============================================================================
TELL1FIRProcessEngine::~TELL1FIRProcessEngine() {}
//=============================================================================
int TELL1FIRProcessEngine::velo_fir_process(int fir_enable, Data in_data, float K0, float K1, float K2)
{
  int err=0;
	int ppx,channel,k;
  //it is not the real coefficient, it is the result of the real
  // coefficient with a multiplication  of 4

	int data_buf[32];
	
	if(fir_enable==0) return(err);

	// K0, K1, K2 set by VeloTELL1FIRFilter, which in turn grabs it from options file
	
	// Here, we have a loop of 4, a loop of 8, then two inner loops of 32. This should do 4*8*(32+32)=2048 strips.

  for(ppx=0;ppx<4;ppx++)
    for(channel=0;channel<8;channel++)   
    {
        // Read a set of 32 strips into our data buffer (which is length 32)
    	for(k=0;k<32;k++)
    	  data_buf[k] = in_data[ppx][channel][k];
    	  
	// For channel 0, simply multiply the buffer val by K0 and stick back into in_data
    	in_data[ppx][channel][0] = (int)(data_buf[0]*K0);
	// For channel 1, find channel1*K0 + channel0>>2 * K1
    	in_data[ppx][channel][1] = (int)((data_buf[1]*K0)+(data_buf[0]*K1));
    	
    	for(k=2;k<32;k++)
    	  in_data[ppx][channel][k] = (int)((data_buf[k]*K0) +(data_buf[k-1]*K1) + (data_buf[k-2]*K2));


	// Read a second set of 32 strips into the data buffer
    	for(k=0;k<32;k++)
    	  data_buf[k] = in_data[ppx][channel][k+32];
    	  
    	in_data[ppx][channel][0+32] = (int)(data_buf[0]*K0);
    	in_data[ppx][channel][1+32] = (int)((data_buf[1]*K0) + (data_buf[0]*K1));
    	
    	for(k=2;k<32;k++)
    	  in_data[ppx][channel][k+32] = (int)((data_buf[k]*K0) + (data_buf[k-1]*K1) + (data_buf[k-2]*K2));
    	
    }

	return(err);
}
//=============================================================================
void TELL1FIRProcessEngine::runFilter()
{
  // prepare data and headers for c-module
  Data data;
  iT dataIT=**data;
  std::copy(inData().begin(), inData().end(), dataIT);
  // run the c-module
  int error=velo_fir_process(processEnable(), data, getK0(), getK1(), getK2());
  // write the subtracted pedestals to out container
  if(!error){    
    const_iT begin=**data;
    const_iT end=begin+dataSize();
    std::copy(begin, end, outData().begin());
    assert(std::equal(begin, end, outData().begin()));
  }
}
//--
//=============================================================================
void TELL1FIRProcessEngine::setK0(const float inK0)
{
  m_K0=inK0;
}
//=============================================================================
void TELL1FIRProcessEngine::setK1(const float inK1)
{
  m_K1=inK1;
}
//=============================================================================
void TELL1FIRProcessEngine::setK2(const float inK2)
{
  m_K2=inK2;
}
//=============================================================================
float TELL1FIRProcessEngine::getK0() const
{
  return ( m_K0 );
}
//=============================================================================
float TELL1FIRProcessEngine::getK1() const
{
  return ( m_K1 );
}
//=============================================================================
float TELL1FIRProcessEngine::getK2() const
{
  return ( m_K2 );
}
//=============================================================================
