// STL
#include<vector>
#include<algorithm>

// local
#include "TELL1Engine/TELL1PedestalProcessEngine.h"

// std
#include <iostream>

//-----------------------------------------------------------------------------
// Implementation file for class : TELL1PedestalProcessEngine
//
// 2007-06-14 : Tomasz Szumlak
//-----------------------------------------------------------------------------

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
TELL1PedestalProcessEngine::TELL1PedestalProcessEngine(  ):
  TELL1Engine(),
  m_scalingMode ( 0 ),
  m_zsEnable ( 0 ),
  m_headerCorrectionFlag ( 0 ),
  m_header ( )
{ 
  this->initTableMembers();
}
//=============================================================================
// Destructor
//=============================================================================
TELL1PedestalProcessEngine::~TELL1PedestalProcessEngine() 
{ }
//=============================================================================
int TELL1PedestalProcessEngine::velo_pedestal_process(int pedestal_enable,
                                int scaling_mode, int zs_enable, 
                                int header_correction_enable,
                                HeaderStrip0 header_correction_value_strip_0,
                                Header header, LinkMask link_mask,
                                LinkPedestal link_pedestal, Data in_data)
{
  int PPFPGA;
  int procChan;
  int strip;
  int beetle;
  int err=0;
  int aLink;
	int header_correction=0;
	int header_zero_threshold=0, header_zero_correction=0;
  int header_one_threshold=0, header_one_correction=0;
  //
	header_zero_threshold=header_correction_value_strip_0[0];
	header_zero_correction=header_correction_value_strip_0[1];
	header_one_threshold=header_correction_value_strip_0[2];
	header_one_correction=header_correction_value_strip_0[3];
  //
  //std::cout<< " first loop " <<std::endl;
	if(pedestal_enable==1){
    for(PPFPGA=0; PPFPGA<4; ++PPFPGA){
			for(procChan=0; procChan<8; ++procChan){
				for(strip=0; strip<64; ++strip){
					if((strip==0)&&(header_correction_enable==1)){
						if(header[PPFPGA][procChan][3]>header_one_threshold){
							header_correction=header_one_correction;
            }else if(header[PPFPGA][procChan][3]<header_zero_threshold){
							header_correction=header_zero_correction;
            }else{
              header_correction = 0;
            }
          }else if((strip==32)&&(header_correction_enable==1)){
						if(header[PPFPGA][procChan][7]>header_one_threshold){
							header_correction=header_one_correction;
            }else if(header[PPFPGA][procChan][7]<header_zero_threshold){
							header_correction = header_zero_correction;
            }else{
              header_correction = 0;
            }
          }else{
						header_correction = 0;
          }
          // header correction
					in_data[PPFPGA][procChan][strip]= 
          in_data[PPFPGA][procChan][strip]-header_correction;
				}
			}
		}
	}
  //
	for(PPFPGA=0; PPFPGA<4; ++PPFPGA){
		if(pedestal_enable==0) continue;
		for(beetle=0; beetle<4; ++beetle){
			for(aLink=0; aLink<4; ++aLink){
				for(strip=0; strip<32; ++strip){
					if((zs_enable==0)||(link_mask[PPFPGA*16+beetle*4+aLink][strip]==1)){
            in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]=0;
          }else if(pedestal_enable==1){
            // changed to (else if) Guido 7.3.2007
            in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]-=
            (link_pedestal[PPFPGA*16+beetle*4+aLink][strip]);
					}
					switch(scaling_mode){
						case 0:
              // printf(RED"\npedestal mode = 0\n");
							if(in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]>127){ 
                in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]=127;
              }else if(in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]<-128){
                in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]=-128;
              }
              // else in_data[i][j][k][4+t] &= 0xFF;
							break;
						case 1:
              // printf(RED"\npedestal mode = 1\n");
							if(in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]>255){
                in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]=127;
              }else if(in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]<-256){
                in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]=-128;
              }else{
                in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]=
                (in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]>>1);
              }
							break;
              //						
						case 2:
              // printf(RED"\npedestal mode = 2\n");
							if(in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]>511){
                in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]=127;
              }else if(in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]<-512){
                in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]=-128;
              }else{
                in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]=
                (in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]>>2);
              }
							break;
              //						
						default:
							in_data[PPFPGA][beetle*2+aLink/2][32*(aLink%2)+strip]=0;
							break;
					}  //switch
				}    //for 32 strip
			}      //for 4 link
		}        //for 4 beetle
  }          //for 4 pp
/* 	for(t=0;t<32;t++) */
/*  printf("ped_sub_data=%4d pedestal=%4d\n",in_data[0][0][t],link_pedestal_sum[0][t]); */
  //
	return ( err );
}
//=============================================================================
void TELL1PedestalProcessEngine::setScalingMode(const int inValue)
{
  m_scalingMode=inValue;
}
//=============================================================================
void TELL1PedestalProcessEngine::setZSEnable(const int inValue)
{
  m_zsEnable=inValue;
}
//=============================================================================
void TELL1PedestalProcessEngine::setHeaderCorrectionFlag(const int inValue)
{
  m_headerCorrectionFlag=inValue;
}
//=============================================================================
void TELL1PedestalProcessEngine::setHeaderCorrectionValues(
                                 const HeaderStrip0Vec& inData)
{
  hiT begin=m_headerCorrectionValueStrip0;
  const_hsIT iTI=inData.begin();
  const_hsIT iTF=inData.end();
  std::copy(iTI, iTF, begin);
  assert(std::equal(iTI, iTF, begin));
}
//=============================================================================
void TELL1PedestalProcessEngine::setHeaders(const EngineVec& inData)
{
  m_header=inData;
}
//=============================================================================
void TELL1PedestalProcessEngine::setLinkPedestal(const LinkPedestalVec& inData)
{
  const_lpIT iTI=inData.begin();
  const_lpIT iTF=inData.end();
  piT begin=*m_linkPedestal;
  std::copy(iTI, iTF, begin);
  assert(std::equal(iTI, iTF, begin));
}
//=============================================================================
void TELL1PedestalProcessEngine::setLinkMask(const LinkMaskVec& inData)
{
  miT begin=*m_linkMask;
  const_lmIT iTI=inData.begin();
  const_lmIT iTF=inData.end();
  std::copy(iTI, iTF, begin);
  assert(std::equal(iTI, iTF, begin));
}
//=============================================================================
void TELL1PedestalProcessEngine::initTableMembers()
{
  tableInit<u_int32_t>((m_headerCorrectionValueStrip0), headerStripSize());
  tableInit<u_int16_t>((*m_linkPedestal), linkPedestalSize());
  tableInit<u_int8_t>((*m_linkMask), linkMaskSize());
}
//=============================================================================
void TELL1PedestalProcessEngine::runSubtraction()
{
  // prepare data and headers for c-module
  Data data;
  tableInit<int>((**data), dataSize());
  Header header;
  iT dataIT=**data;
  iT headerIT=**header;
  std::copy(inData().begin(), inData().end(), dataIT);
  std::copy(m_header.begin(), m_header.end(), headerIT);
  // run the c-module
  int error=velo_pedestal_process(processEnable(),
                                  m_scalingMode,
                                  m_zsEnable, 
                                  m_headerCorrectionFlag,
                                  m_headerCorrectionValueStrip0,
                                  header,
                                  m_linkMask,
                                  m_linkPedestal,
                                  data);
  // write the subtracted pedestals to out container
  if(!error){    
    const_iT begin=**data;
    const_iT end=begin+dataSize();
    std::copy(begin, end, outData().begin());
    assert(std::equal(begin, end, outData().begin()));
  }
}
//--
