// $Id: TELL1UpdateProcessEngine.cpp,v 1.2 2007/08/27 10:44:58 szumlat Exp $
// Include files 
// STL
#include<vector>
#include<algorithm>

// local
#include "TELL1Engine/TELL1UpdateProcessEngine.h"

// std
#include <iostream>

//-----------------------------------------------------------------------------
// Implementation file for class : TELL1UpdateProcessEngine
//
// 2007-08-10 : Tomasz Szumlak
//-----------------------------------------------------------------------------

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
TELL1UpdateProcessEngine::TELL1UpdateProcessEngine(  ):
  TELL1Engine(),
  m_headerCorrectionFlag ( 0 ),
  m_header ( ),
  m_inPedestalSum ( ),
  m_outPedestalSum ( 2304, 0 )
{
  this->initTableMembers();
}
//=============================================================================
// Destructor
//=============================================================================
TELL1UpdateProcessEngine::~TELL1UpdateProcessEngine() {}
//=============================================================================
int TELL1UpdateProcessEngine::velo_pedestal_update(
                              int pedestal_auto_update_enable,
                              int header_correction_enable, 
                              HeaderStrip0 header_correction_value_strip_0,
                              Header header, Data in_data, Data pedestal_sum)
{
	//
  int i,j,k,data_sub_l,header_correction,err = 0;
	int header_zero_threshold,header_zero_correction,header_one_threshold,header_one_correction;

	header_zero_threshold  = header_correction_value_strip_0[0];
	header_zero_correction = header_correction_value_strip_0[1];
	header_one_threshold   = header_correction_value_strip_0[2];
	header_one_correction  = header_correction_value_strip_0[3];

	if (pedestal_auto_update_enable==1)		{
		for(i=0;i<4;i++) //pp
		{
			for(j=0;j<8;j++)//proc channel
			{
				for(k=0;k<64;k++) //strip
				{
					if ((k==0) && (header_correction_enable==1))	{
						if (header[i][j][3] > header_one_threshold) 
							header_correction = header_one_correction;
						else if(header[i][j][3] < header_zero_threshold)
							header_correction = header_zero_correction;
					    else 
                            header_correction = 0;}
					else if ((k==32)	&& (header_correction_enable==1)) 	{
						if (header[i][j][7] > header_one_threshold) 
							header_correction = header_one_correction;
						else if(header[i][j][7] < header_zero_threshold)
							header_correction = header_zero_correction;
						else 
                            header_correction = 0;}
					else
						header_correction = 0;

					data_sub_l = in_data[i][j][k]-(pedestal_sum[i][j][k]>>10) - header_correction; // header correction
					if (data_sub_l > 15) 
						pedestal_sum[i][j][k]=pedestal_sum[i][j][k]+15;  // saturated update
					else if (data_sub_l < -15) 
						pedestal_sum[i][j][k]=pedestal_sum[i][j][k]-15;  // saturated update
					else
						pedestal_sum[i][j][k]=pedestal_sum[i][j][k]+data_sub_l; 
				}
			}
		}


		for(i=0;i<4;i++) //pp
			for(j=8;j<9;j++)//proc channel
			    for(k=0;k<64;k++)//strip
				   pedestal_sum[i][j][k]=0;
	}
	return ( err );
}
//=============================================================================
void TELL1UpdateProcessEngine::setHeaderCorrectionFlag(const int inValue)
{
  m_headerCorrectionFlag=inValue;
}
//=============================================================================
void TELL1UpdateProcessEngine::setHeaderCorrectionValues(
                                 const HeaderStrip0 inData)
{
  hiT begin=m_headerCorrectionValueStrip0;
  const_hiT IT1=inData;
  const_hiT IT2=IT1+headerStripSize();
  std::copy(IT1, IT2, begin);
  assert(std::equal(IT1, IT2, begin));
}
//=============================================================================
void TELL1UpdateProcessEngine::setHeaders(const EngineVec& inData)
{
  m_header=inData;
}
//=============================================================================
void TELL1UpdateProcessEngine::setPedestalSum(const EngineVec& inVec)
{
  m_inPedestalSum=inVec;
}
//=============================================================================
EngineVec& TELL1UpdateProcessEngine::updatedPedestalSum()
{
  return ( m_outPedestalSum );
}
//=============================================================================
void TELL1UpdateProcessEngine::initTableMembers()
{
  tableInit<u_int32_t>((m_headerCorrectionValueStrip0), headerStripSize());
}
//=============================================================================
void TELL1UpdateProcessEngine::runUpdate()
{
  // prepare data and headers for c-module
  Data data;
  Data pedestalSum;
  Header header;
  iT dataIT=**data;
  iT pedIT=**pedestalSum;
  iT headerIT=**header;
  std::copy(inData().begin(), inData().end(), dataIT);
  assert(std::equal(inData().begin(), inData().end(), dataIT));
  std::copy(m_inPedestalSum.begin(), m_inPedestalSum.end(), pedIT);
  assert(std::equal(m_inPedestalSum.begin(), m_inPedestalSum.end(), pedIT));
  std::copy(m_header.begin(), m_header.end(), headerIT);
  assert(std::equal(m_header.begin(), m_header.end(), headerIT));
  // run the c-module
  int error=velo_pedestal_update(processEnable(),
                                 m_headerCorrectionFlag,
                                 m_headerCorrectionValueStrip0,
                                 header,
                                 data,
                                 pedestalSum);
  // write the subtracted pedestals to out container
  if(!error){    
    const_iT begin=**pedestalSum;
    const_iT end=begin+dataSize();
    std::copy(begin, end, updatedPedestalSum().begin());
    assert(std::equal(begin, end, updatedPedestalSum().begin()));
  }
}
//--
