// $Id: TELL1ZSProcessEngine.cpp,v 1.1.1.1 2007/08/26 16:20:13 szumlat Exp $
// Include files 
#include<vector>
#include<algorithm>

// local
#include "TELL1Engine/TELL1ZSProcessEngine.h"

// std
#include <iostream>

//-----------------------------------------------------------------------------
// Implementation file for class : TELL1ZSProcessEngine
//
// 2007-08-12 : Tomasz Szumlak
//-----------------------------------------------------------------------------

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
TELL1ZSProcessEngine::TELL1ZSProcessEngine(  ):
  TELL1Engine(),
  m_producedClusters ( 0 ),
  m_includedADCS ( 0 ),
  m_hitThSize ( sizeof(HitThreshold)/sizeof(u_int8_t) ),
  m_lowThSize ( sizeof(LowThreshold)/sizeof(u_int32_t) ),
  m_sumThSize ( sizeof(SumThreshold)/sizeof(u_int32_t) ),
  m_boundaryStripSize ( sizeof(BoundaryStrip)/sizeof(u_int32_t) ),
  m_clusterSize ( sizeof(TELL1Cluster)/sizeof(u_int16_t) ),
  m_adcSize ( sizeof(TELL1ADC)/sizeof(u_int8_t) ),
  m_rawClusters ( ),
  m_rawADCS ( )
{
  this->initTableMembers();
}
//=============================================================================
// Destructor
//=============================================================================
TELL1ZSProcessEngine::~TELL1ZSProcessEngine() {}
//=============================================================================
void TELL1ZSProcessEngine::initTableMembers()
{
  tableInit<u_int8_t>((**m_hitThresholds), hitThresholdSize());
  tableInit<u_int32_t>((m_lowThresholds), lowThresholdSize());
  tableInit<u_int32_t>((m_sumThresholds), sumThresholdSize());
  tableInit<u_int32_t>((m_boundaryStrips), boundaryStripSize());
  tableInit<u_int16_t>((m_clusters), clusterSize());
  tableInit<u_int8_t>((m_adcs), adcSize());
}
//=============================================================================
int TELL1ZSProcessEngine::velo_zs_process(int zs_enable,
                          HitThreshold hit_threshold,
                          LowThreshold  low_threshold_temp,
                          SumThreshold  sum_temp,
                          BoundaryStrip strip_start_temp,
                          Data in_data, u_int32_t pp_max_clusters,
                          TELL1Cluster cluster, int *clus_numb,
                          TELL1ADC adc_list, int *adc_numb)
{
  	int err = 0;
	int i, j,  t;
	
	int zs_line[69];
	int zs_h_line[69];        //4 left +64 +1 right
	int zs_l_line[69];        //4 left +64 +1 right
	
	int boundary_strip=0;
	
	
	int pp_clus_cnt, pp_adc_cnt;
  
  int hit_threshold_temp[4][9][64];
	
	for(i=0;i<4;i++)
	  for(j=0;j<9;j++)
	    for(t=0;t<64;t++)
	    {
	      if(hit_threshold[i][j][t]>0x7F) hit_threshold_temp[i][j][t] = hit_threshold[i][j][t] -0xFF;
	      else                            hit_threshold_temp[i][j][t] = hit_threshold[i][j][t] ;
	    }

	(*clus_numb)= 0;
	(*adc_numb) =0;
	
	for(i=0; i<4; i++)        //4 pp
	{
		if(zs_enable==0) continue;

		pp_clus_cnt = 0;
		pp_adc_cnt = 0;
		
		for(j=0;j<9;j++)        //9channel
		{
	    //channel 1   substract high threshold and low threshold
	    for(t=0;t<4;t++)      //channel 1
	    {	
	    	if(j==0) { zs_line[t]=0; zs_h_line[t] = -1; zs_l_line[t] = -1; }
	    	else     { zs_h_line[t] = in_data[i][j-1][60+t] - hit_threshold_temp[i][j-1][60+t];
	    		         zs_l_line[t] = in_data[i][j-1][60+t] - low_threshold_temp[i*9+j-1];
	    		         zs_line[t]   = in_data[i][j-1][60+t];
	    		       }
	    }	    
	    for(t=0;t<64;t++)
	    {	
	    	zs_h_line[t+4] = in_data[i][j][t] - hit_threshold_temp[i][j][t]; 
	    	zs_l_line[t+4] = in_data[i][j][t] - low_threshold_temp[i*9+j];
	    	zs_line[t+4]   = in_data[i][j][t];
	    }
	    if(j==8)
	    { 
	    	zs_line[68] = 0;
	    	zs_h_line[68] = -1;
	      zs_l_line[68] = -1;
	      boundary_strip = 1;
	    }
	    else
	    { 
	    	zs_line[68] = in_data[i][j+1][0];
	      zs_h_line[68] = in_data[i][j+1][0] - hit_threshold_temp[i][j+1][0];
	      zs_l_line[68] = in_data[i][j+1][0] - low_threshold_temp[i*9+j+1];
	      boundary_strip = 0;
	    }
    
	    err|=velo_clusterization_process(zs_line, zs_h_line, zs_l_line,
                                       cluster, adc_list, &pp_clus_cnt,
                                       &pp_adc_cnt,pp_max_clusters,
                                       sum_temp[i*9+j],
                                       strip_start_temp[i*9+j],
                                       boundary_strip);
	    if(pp_clus_cnt == (int)pp_max_clusters)   	break;
	    
	    
		}//9channel
	  cluster = cluster+pp_clus_cnt;
	  adc_list = adc_list+pp_adc_cnt;
	  (*clus_numb) += pp_clus_cnt;
		(*adc_numb) += pp_adc_cnt;
	}//4 pp
	
	return(err);
}
//=============================================================================
int TELL1ZSProcessEngine::velo_clusterization_process(int *zs_line,
                                                      int *zs_h_line,
                                                      int *zs_l_line, 
                                                      u_int16_t *cluster,
                                                      u_int8_t *adc_list,
                                                      int *pp_clus_cnt,
                                                      int *pp_adc_cnt,
                                                  u_int32_t pp_max_clusters,
                                                  u_int8_t sum,
                                                  u_int16_t strip_start,
                                                  int boundary_strip)
{
  int err = 0;
	int i;
	int pos;
	int loop_cnt;
	
	int sum_temp;
	int sum_pos;
	int pos_shift;
	
	VELO_CLUSTER_UNIT cluster_unit;
	VELO_ADC_UNIT     adc_unit;
	
	if(boundary_strip ==1) loop_cnt = 69;
	else loop_cnt = 68;

	for(i=4; i<loop_cnt; i++)
	{
		if((zs_h_line[i]<=0)&&(zs_h_line[i-1]>0)&&(zs_h_line[i-2]<=0))   //010
		{
			if((zs_l_line[i]<=0)&&(zs_l_line[i-1]>0)&&(zs_l_line[i-2]>0))//110
		  {
			  sum_temp = zs_line[i-1]+zs_line[i-2];
			  if(sum_temp>sum) cluster_unit.Sepe.CLUSTER_SO = 1;
			  else             cluster_unit.Sepe.CLUSTER_SO = 0;
			  
			  cluster_unit.Sepe.CLUSTER_SIZE = 0;
			  
        pos_shift = 2;
			  sum_pos = (pos_shift-1)*zs_line[i-1]+(pos_shift-2)*zs_line[i-2];
        
        pos = (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192;
			  cluster_unit.Sepe.CLUSTER_POS = (strip_start+i-4-pos_shift+ ((pos>>3)&0x7FF))&0x7FF;
			  cluster_unit.Sepe.CLUSTER_ISP = (pos&0x7);

//printf(RED"\n010>110: zs_line[i-1]= %d zs_line[i-2]= %d pos= %d  isp = %d (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000) = %d \n "
//                   , zs_line[i-1],zs_line[i-2],cluster_unit.Sepe.CLUSTER_POS,cluster_unit.Sepe.CLUSTER_ISP,(sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000) );
//printf(RED"\n010>110:((int)(65536.0F/(float)sum_temp+0.5))=%d ,sum_pos*((int)(65536.0F/(float)sum_temp+0.5)) = %d,(sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000) = %d (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192 = %d \n "
//             ,((int)(65536.0F/(float)sum_temp+0.5)), sum_pos*((int)(65536.0F/(float)sum_temp+0.5)), (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000) , (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192 );

//getchar();

			  *(cluster+(*pp_clus_cnt)) = cluster_unit.All;
			  (*pp_clus_cnt)++;
			  
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-2];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-1];
			  adc_unit.Sepe.ADC_EOC = 1;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  
		  }
		  if((zs_l_line[i]>0)&&(zs_l_line[i-1]>0)&&(zs_l_line[i-2]<=0))//011
		  {
			  sum_temp = zs_line[i-1]+zs_line[i];
			  if(sum_temp>sum) cluster_unit.Sepe.CLUSTER_SO = 1;
			  else             cluster_unit.Sepe.CLUSTER_SO = 0;
			  
			  cluster_unit.Sepe.CLUSTER_SIZE = 0;
			  
        pos_shift = 1;
			  sum_pos = (pos_shift-0)*zs_line[i]+(pos_shift-1)*zs_line[i-1];

        pos = (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192;
			  cluster_unit.Sepe.CLUSTER_POS = (strip_start+i-4-pos_shift+ ((pos>>3)&0x7FF))&0x7FF;
			  cluster_unit.Sepe.CLUSTER_ISP = (pos&0x7);
			  
//printf(RED"\n010>110:((int)(65536.0F/(float)sum_temp+0.5))=%d ,sum_pos*((int)(65536.0F/(float)sum_temp+0.5)) = %d,(sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000) = %d (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192 = %d \n "
//             ,((int)(65536.0F/(float)sum_temp+0.5)), (sum_pos-(int)sum_temp)*((int)(65536.0F/(float)sum_temp+0.5)), ((sum_pos-(int)sum_temp)*((int)(65536.0F/(float)sum_temp+0.5))+0x1000) , ((sum_pos-(int)sum_temp)*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192 );

//printf(RED"\n010>011: zs_line[i]= %d zs_line[i-1]= %d pos= %d  isp = %d\n ", zs_line[i],zs_line[i-1],cluster_unit.Sepe.CLUSTER_POS,cluster_unit.Sepe.CLUSTER_ISP );
//getchar();
			  
			  *(cluster+(*pp_clus_cnt)) = cluster_unit.All;
			  (*pp_clus_cnt)++;
			  
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-1];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i];
			  adc_unit.Sepe.ADC_EOC = 1;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
		  }
		  if((zs_l_line[i]>0)&&(zs_l_line[i-1]>0)&&(zs_l_line[i-2]>0))//111
		  {
			  sum_temp =zs_line[i] + zs_line[i-1]+zs_line[i-2];
			  if(sum_temp>sum) cluster_unit.Sepe.CLUSTER_SO = 1;
			  else             cluster_unit.Sepe.CLUSTER_SO = 0;
			  
			  cluster_unit.Sepe.CLUSTER_SIZE = 1;
        pos_shift = 2;
			  sum_pos = (pos_shift-0)*zs_line[i]+(pos_shift-1)*zs_line[i-1]+(pos_shift-2)*zs_line[i-2];
        pos = (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192;
			  cluster_unit.Sepe.CLUSTER_POS = (strip_start+i-4-pos_shift+ ((pos>>3)&0x7FF))&0x7FF;
			  cluster_unit.Sepe.CLUSTER_ISP = (pos&0x7);
//printf(RED"\n010>110: zs_line[i]= %d zs_line[i-1]= %d zs_line[i-2]= %d pos= %d  isp = %d\n :::a_tmep=%d, b_temp= %d",zs_line[i], zs_line[i-1],zs_line[i-2],cluster_unit.Sepe.CLUSTER_POS,cluster_unit.Sepe.CLUSTER_ISP, a_temp,b_temp );
//getchar();
			  
			  *(cluster+(*pp_clus_cnt)) = cluster_unit.All;
			  (*pp_clus_cnt)++;
			  
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-2];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-1];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i];
			  adc_unit.Sepe.ADC_EOC = 1;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
		  }
		  if((zs_l_line[i]<=0)&&(zs_l_line[i-1]>0)&&(zs_l_line[i-2]<=0))//010
		  {
			  sum_temp = zs_line[i-1];
			  if(sum_temp>sum) cluster_unit.Sepe.CLUSTER_SO = 1;
			  else             cluster_unit.Sepe.CLUSTER_SO = 0;
			  
			  cluster_unit.Sepe.CLUSTER_SIZE = 0;
        pos_shift = 1;
			  sum_pos = (pos_shift-1)*zs_line[i-1];
        pos = (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192;
			  cluster_unit.Sepe.CLUSTER_POS = (strip_start+i-4-pos_shift+ ((pos>>3)&0x7FF))&0x7FF;
			  cluster_unit.Sepe.CLUSTER_ISP = (pos&0x7);
			  
			  *(cluster+(*pp_clus_cnt)) = cluster_unit.All;
			  (*pp_clus_cnt)++;
			  
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-1];
			  adc_unit.Sepe.ADC_EOC = 1;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
		  }
		
		}
		if((zs_h_line[i]<=0)&&(zs_h_line[i-1]>0)&&(zs_h_line[i-2]>0)&&(zs_h_line[i-3]<=0))   //0110
		{
			if((zs_l_line[i]<=0)&&(zs_l_line[i-1]>0)&&(zs_l_line[i-2]>0)&&(zs_l_line[i-3]>0))//1110
		  {
			  sum_temp = zs_line[i-1]+zs_line[i-2]+zs_line[i-3];
			  if(sum_temp>sum) cluster_unit.Sepe.CLUSTER_SO = 1;
			  else             cluster_unit.Sepe.CLUSTER_SO = 0;
			  
			  cluster_unit.Sepe.CLUSTER_SIZE = 1;
        pos_shift = 3;
			  sum_pos = (pos_shift-1)*zs_line[i-1]+(pos_shift-2)*zs_line[i-2]+(pos_shift-3)*zs_line[i-3];
        pos = (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192;
			  cluster_unit.Sepe.CLUSTER_POS = (strip_start+i-4-pos_shift+ ((pos>>3)&0x7FF))&0x7FF;

			  cluster_unit.Sepe.CLUSTER_ISP = (pos&0x7);
			  
			  *(cluster+(*pp_clus_cnt)) = cluster_unit.All;
			  (*pp_clus_cnt)++;
			  
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-3];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-2];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-1];
			  adc_unit.Sepe.ADC_EOC = 1;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  
		  }
		  if((zs_l_line[i]>0)&&(zs_l_line[i-1]>0)&&(zs_l_line[i-2]>0)&&(zs_l_line[i-3]<=0))//0111
		  {
			  sum_temp =zs_line[i-2]+zs_line[i-1]+zs_line[i];
			  if(sum_temp>sum) cluster_unit.Sepe.CLUSTER_SO = 1;
			  else             cluster_unit.Sepe.CLUSTER_SO = 0;
			  
			  cluster_unit.Sepe.CLUSTER_SIZE = 1;
        pos_shift = 2;
			  sum_pos = (pos_shift-0)*zs_line[i]+(pos_shift-1)*zs_line[i-1]+(pos_shift-2)*zs_line[i-2];
			  
        pos = (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192;
			  cluster_unit.Sepe.CLUSTER_POS = (strip_start+i-4-pos_shift+ ((pos>>3)&0x7FF))&0x7FF;
			  cluster_unit.Sepe.CLUSTER_ISP = (pos&0x7);
//printf(RED"\n010>110: zs_line[i]= %d zs_line[i-1]= %d zs_line[i-2]= %d pos= %d  isp = %d\n :::a_tmep=%d,b_temp=%d",zs_line[i], zs_line[i-1],zs_line[i-2],cluster_unit.Sepe.CLUSTER_POS,cluster_unit.Sepe.CLUSTER_ISP, a_temp,b_temp );
//getchar();
			  
			  *(cluster+(*pp_clus_cnt)) = cluster_unit.All;
			  (*pp_clus_cnt)++;
			  
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-2];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-1];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i];
			  adc_unit.Sepe.ADC_EOC = 1;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
		  }
		  if((zs_l_line[i]>0)&&(zs_l_line[i-1]>0)&&(zs_l_line[i-2]>0)&&(zs_l_line[i-3]>0))//1111
		  {
			  sum_temp =zs_line[i] + zs_line[i-1]+zs_line[i-2]+zs_line[i-3];
			  if(sum_temp>sum) cluster_unit.Sepe.CLUSTER_SO = 1;
			  else             cluster_unit.Sepe.CLUSTER_SO = 0;
			  
			  cluster_unit.Sepe.CLUSTER_SIZE = 1;
        pos_shift = 3;
			  sum_pos = (pos_shift-0)*zs_line[i]+(pos_shift-1)*zs_line[i-1]+(pos_shift-2)*zs_line[i-2]+(pos_shift-3)*zs_line[i-3];
        pos = (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192;
			  cluster_unit.Sepe.CLUSTER_POS = (strip_start+i-4-pos_shift+ ((pos>>3)&0x7FF))&0x7FF;
			  cluster_unit.Sepe.CLUSTER_ISP = (pos&0x7);

//printf(RED"\n010>110: zs_line[i]= %X zs_line[i-1]= %X zs_line[i-2]= %X zs_line[i-3]= %X pos= %X  isp = %d \n :::sum_temp = %X, sum_pos = %X  a_tmep=%X"
//           ,zs_line[i], zs_line[i-1],zs_line[i-2],zs_line[i-3],cluster_unit.Sepe.CLUSTER_POS,cluster_unit.Sepe.CLUSTER_ISP,(int)sum_temp,sum_pos,a_temp);
//getchar();
			  
			  *(cluster+(*pp_clus_cnt)) = cluster_unit.All;
			  (*pp_clus_cnt)++;
			  
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-3];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-2];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-1];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i];
			  adc_unit.Sepe.ADC_EOC = 1;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
		  }
		  if((zs_l_line[i]<=0)&&(zs_l_line[i-1]>0)&&(zs_l_line[i-2]>0)&&(zs_l_line[i-3]<=0))//0110
		  {
			  sum_temp = zs_line[i-1]+zs_line[i-2];
			  if(sum_temp>sum) cluster_unit.Sepe.CLUSTER_SO = 1;
			  else             cluster_unit.Sepe.CLUSTER_SO = 0;
			  
			  cluster_unit.Sepe.CLUSTER_SIZE = 0;
			  
        pos_shift = 2;
			  sum_pos = (pos_shift-1)*zs_line[i-1]+(pos_shift-2)*zs_line[i-2];
        pos = (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192;
			  cluster_unit.Sepe.CLUSTER_POS = (strip_start+i-4-pos_shift+ ((pos>>3)&0x7FF))&0x7FF;

			  cluster_unit.Sepe.CLUSTER_ISP = (pos&0x7);
			  
			  *(cluster+(*pp_clus_cnt)) = cluster_unit.All;
			  (*pp_clus_cnt)++;
			  
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-2];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-1];
			  adc_unit.Sepe.ADC_EOC = 1;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
		  }
		
		}
		if((zs_h_line[i]<=0)&&(zs_h_line[i-1]>0)&&(zs_h_line[i-2]>0)&&(zs_h_line[i-3]>0)&&(zs_h_line[i-4]<=0))   //01110
		{
			  sum_temp = zs_line[i-1]+zs_line[i-2]+zs_line[i-3];
			  if(sum_temp>sum) cluster_unit.Sepe.CLUSTER_SO = 1;
			  else             cluster_unit.Sepe.CLUSTER_SO = 0;
			  
			  cluster_unit.Sepe.CLUSTER_SIZE = 1;
			  
        pos_shift = 3;
			  sum_pos = (pos_shift-1)*zs_line[i-1]+(pos_shift-2)*zs_line[i-2]+(pos_shift-3)*zs_line[i-3];
        pos = (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192;
			  cluster_unit.Sepe.CLUSTER_POS = (strip_start+i-4-pos_shift+ ((pos>>3)&0x7FF))&0x7FF;

			  cluster_unit.Sepe.CLUSTER_ISP = (pos&0x7);
			  
//			  cluster_unit.Sepe.CLUSTER_POS = strip_start+i-8+ (sum_pos/sum_temp);
//			  cluster_unit.Sepe.CLUSTER_ISP = ((sum_pos*8)/sum_temp)%8;
			  *(cluster+(*pp_clus_cnt)) = cluster_unit.All;
			  (*pp_clus_cnt)++;
			  
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-3];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-2];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-1];
			  adc_unit.Sepe.ADC_EOC = 1;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
		
		}
		if((zs_h_line[i]>0)&&(zs_h_line[i-1]>0)&&(zs_h_line[i-2]>0)&&(zs_h_line[i-3]>0)&&(zs_h_line[i-4]<=0))   //01111
		{
			  sum_temp = zs_line[i]+zs_line[i-1]+zs_line[i-2]+zs_line[i-3];
			  if(sum_temp>sum) cluster_unit.Sepe.CLUSTER_SO = 1;
			  else             cluster_unit.Sepe.CLUSTER_SO = 0;
			  
			  cluster_unit.Sepe.CLUSTER_SIZE = 1;
			  
        pos_shift = 3;
			  sum_pos = (pos_shift-0)*zs_line[i]+(pos_shift-1)*zs_line[i-1]+(pos_shift-2)*zs_line[i-2]+(pos_shift-3)*zs_line[i-3];
        pos = (sum_pos*((int)(65536.0F/(float)sum_temp+0.5))+0x1000)/8192;
			  cluster_unit.Sepe.CLUSTER_POS = (strip_start+i-4-pos_shift+ ((pos>>3)&0x7FF))&0x7FF;
			  cluster_unit.Sepe.CLUSTER_ISP = (pos&0x7);
			  
			  *(cluster+(*pp_clus_cnt)) = cluster_unit.All;
			  (*pp_clus_cnt)++;
			  
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-3];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-2];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i-1];
			  adc_unit.Sepe.ADC_EOC = 0;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
			  adc_unit.Sepe.ADC_VALUE = zs_line[i];
			  adc_unit.Sepe.ADC_EOC = 1;
			  *(adc_list+(*pp_adc_cnt)) = adc_unit.All;
			  (*pp_adc_cnt)++;
		
		}
		if((zs_h_line[i]>0)&&(zs_h_line[i-1]>0)&&(zs_h_line[i-2]>0)&&(zs_h_line[i-3]>0)&&(zs_h_line[i-4]>0))   //11111
		{
			  zs_h_line[i-1] = -1;
		}
		if((*pp_clus_cnt)==(int)pp_max_clusters) return(err);
		
	}
	
	return(err);
}
//=============================================================================
size_t TELL1ZSProcessEngine::hitThresholdSize() const
{
  return ( m_hitThSize );
}
//=============================================================================
size_t TELL1ZSProcessEngine::lowThresholdSize() const
{
  return ( m_lowThSize );
}
//=============================================================================
size_t TELL1ZSProcessEngine::sumThresholdSize() const
{
  return ( m_sumThSize );
}
//=============================================================================
size_t TELL1ZSProcessEngine::boundaryStripSize() const
{
  return ( m_boundaryStripSize );
}
//=============================================================================
size_t TELL1ZSProcessEngine::clusterSize() const
{
  return ( m_clusterSize );
}
//=============================================================================
size_t TELL1ZSProcessEngine::adcSize() const
{
  return ( m_adcSize );
}
//=============================================================================
void TELL1ZSProcessEngine::setHitThresholds(const HitThresholdVec& inData)
{
  const_hthIT iTI=inData.begin();
  const_hthIT iTF=inData.end();
  miT begin=**m_hitThresholds;
  std::copy(iTI, iTF, begin);
  assert(std::equal(iTI, iTF, begin));
}
//=============================================================================
void TELL1ZSProcessEngine::setLowThresholds(const Thresholds& inData)
{
  const_thIT iTI=inData.begin();
  const_thIT iTF=inData.end();
  hiT begin=m_lowThresholds;
  std::copy(iTI, iTF, begin);
  assert(std::equal(iTI, iTF, begin));
}
//=============================================================================
void TELL1ZSProcessEngine::setSumThresholds(const Thresholds& inData)
{
  const_thIT iTI=inData.begin();
  const_thIT iTF=inData.end();
  hiT begin=m_sumThresholds;
  std::copy(iTI, iTF, begin);
  assert(std::equal(iTI, iTF, begin));
}
//=============================================================================
void TELL1ZSProcessEngine::setBoundaryStrips(const BoundaryStripVec& inData)
{
  const_bsIT iTI=inData.begin();
  const_bsIT iTF=inData.end();
  hiT begin=m_boundaryStrips;
  std::copy(iTI, iTF, begin);
  assert(std::equal(iTI, iTF, begin));
}
//=============================================================================
void TELL1ZSProcessEngine::flushMemory()
{
  tableInit<u_int16_t>(m_clusters, clusterSize());
  tableInit<u_int8_t>(m_adcs, adcSize());
  m_producedClusters=0;
  m_includedADCS=0;
  m_rawClusters.clear();
  m_rawADCS.clear();
}
//=============================================================================
TELL1ClusterVec TELL1ZSProcessEngine::outClusters()
{
  return ( m_rawClusters );
}
//=============================================================================
TELL1ADCVec TELL1ZSProcessEngine::outADCS()
{
  return ( m_rawADCS );
}
//=============================================================================
void TELL1ZSProcessEngine::runZeroSuppression()
{
  // prepare data for the c-module
  flushMemory();
  Data data;
  tableInit<int>((**data), dataSize());
  iT dataIT=**data;
  std::copy(inData().begin(), inData().end(), dataIT);
  assert(std::equal(inData().begin(), inData().end(), dataIT));
  int error=velo_zs_process(processEnable(), 
                            m_hitThresholds,
                            m_lowThresholds,
                            m_sumThresholds,
                            m_boundaryStrips,
                            data,
                            PP_MAX_CLUSTERS,
                            m_clusters,
                            &m_producedClusters,
                            m_adcs,
                            &m_includedADCS);
  // return created clusters and added adcs
  if(!error){
    for(int cluster=0; cluster<m_producedClusters; ++cluster){
      m_rawClusters.push_back(m_clusters[cluster]);
    }
    for(int adc=0; adc<m_includedADCS; ++adc){
      m_rawADCS.push_back(m_adcs[adc]);
    }
  }
}
//--
