
void badChannel()
{
  cout << "Create a list of bad channels according to noise performance, and make noise/pedestal plots" << endl;
  cout << "   badChannel(char inFile[])" << endl;
  cout << "   badChannel(char inFile[], int cableConfiguration)" << endl;
  cout << "   badChannel(char inFile[], int cableConfiguration, int printSelect)" << endl;
  cout << "   badChannel(char inFile[], int cableConfiguration, int printSelect, int maxPrintOut)" << endl;
  cout << "      cableConfiguration (2):  module/cable configurations" << endl;
  cout << "      printSelect (0): bad channel output options" << endl;
  cout << "          0: No bad channel output, 1: Summary, 2: Only new bad list," << endl;
  cout << "          3: All cern bad list, 4: Only Liverpool bad list, 5: All bad list" << endl;
  cout << "      maxPrintOut (5): maximum printout per module, -1 means all" << endl;
}

void badChannel(char inFile[])
{
  int cableConfiguration = 4;
  int printSelect = 0; // 1: Summary, 2: Only new bad list, 3: All cern bad list, 4: Only Liverpool bad list, 5: All bad list
  int maxPrintOut = 5; // maximum printout per module, -1 means all
  badChannel(inFile, cableConfiguration, printSelect, maxPrintOut);
}

void badChannel(char inFile[], int cableConfiguration)
{
  int printSelect = 0;
  int maxPrintOut = 5;
  badChannel(inFile, cableConfiguration, printSelect, maxPrintOut);
}

void badChannel(char inFile[], int cableConfiguration, int printSelect)
{
  int maxPrintOut = 5;
  badChannel(inFile, cableConfiguration, printSelect, maxPrintOut);
}

void badChannel(char inFile[], int cableConfiguration, int printSelect, int maxPrintOut)
{
  Float_t noiseLowLimit = 0.4;
  Float_t noiseUpLimit  = 0.4;
  Float_t noiseLowConst = 1.5;
  Float_t noiseUpConst  = 4;

  char rootFile[400];
  char cableConfig[400];
  sprintf(rootFile,"%s",inFile);
  sprintf(cableConfig, "Cable Config. %d", cableConfiguration);
  cout << " ===== " << cableConfig << " =====" << endl;
  
  TString inFileTString=TString(inFile);
  Size_t dotPosition =inFileTString.Index(".root");
  TString titleTString = inFileTString(0,dotPosition);
  //  char title[]= titleTString.Data();
  char title[400];
  strcpy(title, titleTString.Data());
  cout << " =   for " << title << endl;

  // Select the module/cable configuration
  int nTELL1s = 12;
  int TELL1IDs[12];
  int ModuleIDs[12];
  int ModuleType[12];

  int TELL1IDs1[]  = {21,30,29,62,61,23,51,50,49,27,25,24};
  int ModuleIDs1[] = {26,26,29,29,23,23,31,31,30,30,28,28};
  int ModuleType1[] = {0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0};//r=0, phi=1

  int TELL1IDs2[]  = {30,21,29,62,61,23,51,50,49,27,25,24};
  int ModuleIDs2[] = {29,29,31,31,37,37,27,27,24,24,28,28};
  int ModuleType2[] = {1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0};

  int TELL1IDs3[]  = {21,30,29,62,61,23,51,50,49,27,25,24};
  int ModuleIDs3[] = {26,26,29,29,23,23,31,31,30,30,27,27};
  int ModuleType3[] = {0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0};

  int TELL1IDs4[]  =  {30,21,29,62,61,23,51,50,49,27,25,24};
  int ModuleIDs4[] =  {24,24,29,29,37,37,31,31,30,30,27,27};
  int ModuleType4[] =  {1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0};
  


  for ( int i = 0; i < 12; i++) {
    if ( cableConfiguration == 1 ) {
      TELL1IDs[i]  = TELL1IDs1[i];
      ModuleIDs[i] = ModuleIDs1[i];
      ModuleType[i]= ModuleType1[i];
    } else if ( cableConfiguration == 2 ) {
      TELL1IDs[i]  = TELL1IDs2[i];
      ModuleIDs[i] = ModuleIDs2[i];
      ModuleType[i]= ModuleType2[i];
    } else if ( cableConfiguration == 3 ) {
      TELL1IDs[i]  = TELL1IDs3[i];
      ModuleIDs[i] = ModuleIDs3[i];
      ModuleType[i]= ModuleType3[i];
    } else {
      TELL1IDs[i]  = TELL1IDs4[i];
      ModuleIDs[i] = ModuleIDs4[i];
      ModuleType[i]= ModuleType4[i];
    }
  }

  cout << "# ====  ROOT file:   " << rootFile << endl;
  cout << "# ====  Bad channel standard: totNoise < average-" << noiseLowLimit 
       << "#   cmsNoise > average+" << noiseUpLimit << endl;
  cout << "# --------------------------------------------------------" << endl;

  gStyle->SetOptStat(0); // name only
  gStyle->SetOptFit(0);

  // Date
  TDatime now;
  now.Get();
  TPaveLabel *pDate = new TPaveLabel(0,0,0.3,1,now.AsSQLString(),"NDC");
  pDate->SetBorderSize(0);
  pDate->SetTextFont(32);
  pDate->SetFillColor(0);

  // Cable configuration version
  TPaveLabel *pConfig = new TPaveLabel( 0.35, 0, 0.5, 1, cableConfig,"NDC");
  pConfig->SetBorderSize(0);
  pConfig->SetTextFont(32);
  pConfig->SetFillColor(0); 

  // Title
  TPaveLabel *pTitle = new TPaveLabel( 0.55, 0, 1, 1, title, "NDC");
  pTitle->SetBorderSize(0);
  pTitle->SetTextFont(32);
  pTitle->SetFillColor(0); 

  gROOT->SetStyle("Plain");
  float border = 0.98;

  // All canvas
  TCanvas *c2 = new TCanvas("c2", "Upstream noise", 10, 10, 700, 900);
  TPad *pPad2 = new TPad("pPad2", "Title Pad", 0.0, border, 1.0, 1.0);
  TPad *mPad2 = new TPad("mPad2", "Main Pad", 0.0, 0.0, 1.0, border);
  pPad2->Draw();pPad2->cd();pDate->Draw();pConfig->Draw();pTitle->Draw();
  c2->cd(); mPad2->Draw(); mPad2->Divide(2,3);
  
  TCanvas *c3 = new TCanvas("c3","Downstream noise",10, 10, 700, 900);
  TPad *pPad3 = new TPad("pPad3", "Title Pad", 0.0, border, 1.0, 1.0);
  TPad *mPad3 = new TPad("mPad3", "Main Pad", 0.0, 0.0, 1.0, border);
  pPad3->Draw();pPad3->cd();pDate->Draw();pConfig->Draw();pTitle->Draw();
  c3->cd(); mPad3->Draw(); mPad3->Divide(2,3);
  
  TCanvas *c4 = new TCanvas("c4","Upstream swnoise",10, 10, 700, 900);
  TPad *pPad4 = new TPad("pPad4", "Title Pad", 0.0, border, 1.0, 1.0);
  TPad *mPad4 = new TPad("mPad4", "Main Pad", 0.0, 0.0, 1.0, border);
  pPad4->Draw();pPad4->cd();pDate->Draw();pConfig->Draw();pTitle->Draw();
  c4->cd(); mPad4->Draw(); mPad4->Divide(2,3);
  
  TCanvas *c5 = new TCanvas("c5","Downstream swnoise",10, 10, 700, 900);
  TPad *pPad5 = new TPad("pPad5", "Title Pad", 0.0, border, 1.0, 1.0);
  TPad *mPad5 = new TPad("mPad5", "Main Pad", 0.0, 0.0, 1.0, border);
  pPad5->Draw();pPad5->cd();pDate->Draw();pConfig->Draw();pTitle->Draw();
  c5->cd(); mPad5->Draw(); mPad5->Divide(2,3);
  
//   TCanvas *c6 = new TCanvas("c6","Upstream signal",10, 10, 700, 900);
//   TPad *pPad6 = new TPad("pPad6", "Title Pad", 0.0, border, 1.0, 1.0);
//   TPad *mPad6 = new TPad("mPad6", "Main Pad", 0.0, 0.0, 1.0, border);
//   pPad6->Draw();pPad6->cd();pDate->Draw();pConfig->Draw();pTitle->Draw();
//   c6->cd(); mPad6->Draw(); mPad6->Divide(2,3);
  
//   TCanvas *c7 = new TCanvas("c7","Downstream signal",10, 10, 700, 900);
//   TPad *pPad7 = new TPad("pPad7", "Title Pad", 0.0, border, 1.0, 1.0);
//   TPad *mPad7 = new TPad("mPad7", "Main Pad", 0.0, 0.0, 1.0, border);
//   pPad7->Draw();pPad7->cd();pDate->Draw();pConfig->Draw();pTitle->Draw();
//   c7->cd(); mPad7->Draw(); mPad7->Divide(2,3);

  TCanvas *c8 = new TCanvas("c8","Upstream raw pedestals",10, 10, 700, 900);
  TPad *pPad8 = new TPad("pPad8", "Title Pad", 0.0, border, 1.0, 1.0);
  TPad *mPad8 = new TPad("mPad8", "Main Pad", 0.0, 0.0, 1.0, border);
  pPad8->Draw();pPad8->cd();pDate->Draw();pConfig->Draw();pTitle->Draw();
  c8->cd(); mPad8->Draw(); mPad8->Divide(2,3);
  
  TCanvas *c9 = new TCanvas("c9","Downstream raw pedestals",10, 10, 700, 900);
  TPad *pPad9 = new TPad("pPad9", "Title Pad", 0.0, border, 1.0, 1.0);
  TPad *mPad9 = new TPad("mPad9", "Main Pad", 0.0, 0.0, 1.0, border);
  pPad9->Draw();pPad9->cd();pDate->Draw();pConfig->Draw();pTitle->Draw();
  c9->cd(); mPad9->Draw(); mPad9->Divide(2,3);


  //module 18R only
  int iLivList_m18r[] = {2046,1981,1972,1944,1943,1940,1935,1873,341,1364,1360,1357,
                         336,337,333,332,297,213,887,746,31,30,10,0};

  int iLivList_m18p[] = {0}; // not available

	//module 21
  int iLivList_m21r[] = {74, 376,400,420,424,444,468,484,502,537,795,832,856,968,1023,
           1062,1165,1380,1521,1531,1534,1631,1633,1634,1641,1762,1950,2029,2031,2047};
  int iLivList_m21p[] = {102,103,129,377,486,578,670,870,872,1067,1082,1212,1218,
                         1249,1353,1355,1356,1450,1704,1839};
  //module 23
  int iLivList_m23r[] = {405, 1066, 1067, 1193, 1194, 1195, 1365, 1366, 1554, 
                         1673, 1674, 1675, 1799};
  int iLivList_m23p[] = {30, 31, 32, 234, 495, 741, 742, 743, 744, 745, 746, 747, 748, 
                         1005, 1130, 1151, 1280, 1853, 2011};
  //module 24
  int iLivList_m24r[] = {49,150,420,422,427,428,429,430,431,1373,1403,1748,1924,1964};
  int iLivList_m24p[] = {1379,1422,2026,2027,2028}; 

  //module 26
  int iLivList_m26r[] = {1039, 1477, 2035};
  int iLivList_m26p[] = {82, 505, 560, 587, 658, 1694, 1804, 1957, 1998};

  //module 27
  int iLivList_m27r[] = {473,509,995,1052,1315,1383,1400,1485,1635,1636,1644,
                         1646,1725,1844,1845,1896,2035,2042};
  int iLivList_m27p[] = {34,41,59,156,191,194,240,246,247,278,582,607,882,1153,
                         1159,1169,1523,1524,1933};
  //module 28
  int iLivList_m28p[] = {0,533,1023,1116,1135,1410,1469,1749,1764,1779,1844,
                         1845,1984,2026,2027,2039};
  int iLivList_m28r[] = {169,170,327,1173,1209,1297,1321,1326,1329,1419,1525,
                         1851,1852,1953,1996,2004};

  //module 29
  int iLivList_m29r[] = {24, 26, 461, 489, 519, 858, 859, 905, 919, 992, 1352, 
                    1518, 1522, 1535, 1672, 1882, 1924, 1925, 1971, 1997};
  int iLivList_m29p[] = {0, 285, 356, 357, 358, 359, 360, 361, 529, 1693, 1742};

  //module 30
  int iLivList_m30r[] = {76  , 131 , 243 , 406 , 495 , 888 , 890 , 977 , 1347,
                         1398, 1404, 1418, 1489, 1870, 1952};
  int iLivList_m30p[] = {172 , 409 , 421 , 490 , 530 , 1100, 1663, 1743, 1854};

  //module 31
  int iLivList_m31r[] = {1,147,155,165 ,599,783,818,1028,1029,1241,1243,1269,
                         1271,1518,1566,1606,1774,1950,2007};
  int iLivList_m31p[] = {327, 418, 464, 564, 565, 566, 870, 993, 1213, 1265,
                         1510, 1668, 1755, 1886, 1912};
  //module 37
  int iLivList_m37r[] = {157, 405, 683, 770, 878, 961, 1274, 1275, 1286, 1287, 
                         1288, 1449, 1529, 1685, 1697, 2033, 2038}; 
  int iLivList_m37p[] = {457, 683, 684, 766, 1001, 1089, 1090, 1091, 1092,
                         1093, 1094, 1452, 1472, 1597, 2001};

  int *ilivList;

  // Open ROOT file and access (sub)directories
  TFile *ff=TFile::Open(rootFile);
  if ( ff == NULL ) {
		cout << "#  No such file!" << endl;
		return;
  }

  TDirectory *tdVetra = ff->Get("Vetra");
  if ( tdVetra == NULL ) {
		cout << "#  No Vetra directory " << endl;
		return;
  }


  char module[200];
  for (int iTELL1=0; iTELL1<nTELL1s; iTELL1++){
  //  for (int iTELL1=2; iTELL1<3; iTELL1++){

    int nliv = 0;
    if ( ModuleType[iTELL1] == 0 ) {
      if ( ModuleIDs[iTELL1] == 21 ) ilivList = iLivList_m21r;
      if ( ModuleIDs[iTELL1] == 23 ) ilivList = iLivList_m23r;
      if ( ModuleIDs[iTELL1] == 24 ) ilivList = iLivList_m24r;
      if ( ModuleIDs[iTELL1] == 26 ) ilivList = iLivList_m26r;
      if ( ModuleIDs[iTELL1] == 27 ) ilivList = iLivList_m27r;
      if ( ModuleIDs[iTELL1] == 28 ) ilivList = iLivList_m28r;
      if ( ModuleIDs[iTELL1] == 29 ) ilivList = iLivList_m29r;
      if ( ModuleIDs[iTELL1] == 30 ) ilivList = iLivList_m30r;
      if ( ModuleIDs[iTELL1] == 31 ) ilivList = iLivList_m31r;
      if ( ModuleIDs[iTELL1] == 37 ) ilivList = iLivList_m37r;
      if ( ModuleIDs[iTELL1] == 21 ) nliv = sizeof(iLivList_m21r)/sizeof(*iLivList_m21r);
      if ( ModuleIDs[iTELL1] == 23 ) nliv = sizeof(iLivList_m23r)/sizeof(*iLivList_m23r);
      if ( ModuleIDs[iTELL1] == 24 ) nliv = sizeof(iLivList_m24r)/sizeof(*iLivList_m24r);
      if ( ModuleIDs[iTELL1] == 26 ) nliv = sizeof(iLivList_m26r)/sizeof(*iLivList_m26r);
      if ( ModuleIDs[iTELL1] == 27 ) nliv = sizeof(iLivList_m27r)/sizeof(*iLivList_m27r);
      if ( ModuleIDs[iTELL1] == 28 ) nliv = sizeof(iLivList_m28r)/sizeof(*iLivList_m28r);
      if ( ModuleIDs[iTELL1] == 29 ) nliv = sizeof(iLivList_m29r)/sizeof(*iLivList_m29r);
      if ( ModuleIDs[iTELL1] == 30 ) nliv = sizeof(iLivList_m30r)/sizeof(*iLivList_m30r);
      if ( ModuleIDs[iTELL1] == 31 ) nliv = sizeof(iLivList_m31r)/sizeof(*iLivList_m31r);
      if ( ModuleIDs[iTELL1] == 37 ) nliv = sizeof(iLivList_m37r)/sizeof(*iLivList_m37r);
    } else {
      if ( ModuleIDs[iTELL1] == 21 ) ilivList = iLivList_m21p;
      if ( ModuleIDs[iTELL1] == 23 ) ilivList = iLivList_m23p;
      if ( ModuleIDs[iTELL1] == 24 ) ilivList = iLivList_m24p;
      if ( ModuleIDs[iTELL1] == 26 ) ilivList = iLivList_m26p;
      if ( ModuleIDs[iTELL1] == 27 ) ilivList = iLivList_m27p;
      if ( ModuleIDs[iTELL1] == 28 ) ilivList = iLivList_m28p;
      if ( ModuleIDs[iTELL1] == 29 ) ilivList = iLivList_m29p;
      if ( ModuleIDs[iTELL1] == 30 ) ilivList = iLivList_m30p;
      if ( ModuleIDs[iTELL1] == 31 ) ilivList = iLivList_m31p;
      if ( ModuleIDs[iTELL1] == 37 ) ilivList = iLivList_m37p;
      if ( ModuleIDs[iTELL1] == 21 ) nliv = sizeof(iLivList_m21p)/sizeof(*iLivList_m21p);
      if ( ModuleIDs[iTELL1] == 23 ) nliv = sizeof(iLivList_m23p)/sizeof(*iLivList_m23p);
      if ( ModuleIDs[iTELL1] == 24 ) nliv = sizeof(iLivList_m24p)/sizeof(*iLivList_m24p);
      if ( ModuleIDs[iTELL1] == 26 ) nliv = sizeof(iLivList_m26p)/sizeof(*iLivList_m26p);
      if ( ModuleIDs[iTELL1] == 27 ) nliv = sizeof(iLivList_m27p)/sizeof(*iLivList_m27p);
      if ( ModuleIDs[iTELL1] == 28 ) nliv = sizeof(iLivList_m28p)/sizeof(*iLivList_m28p);
      if ( ModuleIDs[iTELL1] == 29 ) nliv = sizeof(iLivList_m29p)/sizeof(*iLivList_m29p);
      if ( ModuleIDs[iTELL1] == 30 ) nliv = sizeof(iLivList_m30p)/sizeof(*iLivList_m30p);
      if ( ModuleIDs[iTELL1] == 31 ) nliv = sizeof(iLivList_m31p)/sizeof(*iLivList_m31p);
      if ( ModuleIDs[iTELL1] == 37 ) nliv = sizeof(iLivList_m37p)/sizeof(*iLivList_m37p);
    }

    int tell1Id=TELL1IDs[iTELL1];
    if ( ModuleType[iTELL1] == 0 ) {
      sprintf(module,"TELL1 %d M%dR",tell1Id,ModuleIDs[iTELL1]);
    } else {
      sprintf(module,"TELL1 %d M%dP",tell1Id,ModuleIDs[iTELL1]);
    }
    
    char hybridType[20];
    if (ModuleType[iTELL1]){
      sprintf(hybridType,"p");
    }
    else{
      sprintf(hybridType,"r");
    }
    
    cout << "==== TELL1 " << TELL1IDs[iTELL1]
         << "     Module " << ModuleIDs[iTELL1]
         << hybridType << endl;
    
    char dirName[50];
    //    sprintf(dirName, "TELL1_%d_ADC", tell1Id );
    sprintf(dirName, "ADCTELL1%d", tell1Id );
    TDirectory *tdADC = (TDirectory*) tdVetra->Get(dirName);
    if ( tdADC == NULL ) {
  		cout << "#  Directory Vetra/" << dirName << " does not exist" << endl;
    }
    
    sprintf(dirName, "CommonModeTELL1%d", tell1Id );
    TDirectory *tdCommonMode =  (TDirectory*) tdVetra->Get(dirName);
    if ( tdCommonMode == NULL ) {
      cout << "#  Directory Vetra/" << dirName << " does not exist" << endl;
    }
    
    sprintf(dirName, "PedAndNoiseTELL1%d", tell1Id );
    TDirectory *tdPedAndNoise = (TDirectory*) tdVetra->Get(dirName);
    if ( tdADC == NULL ) {
      cout << "#  Directory Vetra/" << dirName << " does not exist" << endl;
    }
    
    if ( tdADC != NULL && tdCommonMode != NULL && tdADC != NULL){
      // Some variables
      int i, j;
      int iCard; // 0-3
      int iChip, chipId;  // 0-3, 0-15
      int iLink, linkId; // 0-3, 0-63
      int iChannel; // 0-127
      int channelEid; // channel Electronics ID, 0-2047
      int channelSid; // channel Software strip ID, 0-2047
      
      char name[50];
      
      int pad;
      char row[10];
      int sector;
      int strip;
      int innerStrip;
      int outerStrip;
      int sector;
      int strip;
      char pa[10];
      int papad;
      int pachan;
      int chip;
      int swstrip;
      int chipchan;
      int swStripList[2049];
      
      Bool_t linkUsed[64];
      Double_t rawNoise[64];
      Double_t sqrawNoise[64];
      Double_t cmsNoise[64];
      Double_t sqcmsNoise[64];
      Double_t rawNoiseRms[64];
      Double_t cmsNoiseRms[64];
      Float_t noisycutused[64];
      Float_t deadcutused[64];
      
      Double_t deadCut;
      Double_t noisyCut;
      char* livnoisedead[2];
      char* noisedead[5];
      int cernBadChannel[2048];
      int livBadChannel[2048];
      livnoisedead[0] = "lpoolOK";
      livnoisedead[1] = "lpoolBAD";
      noisedead[0] = "OK";
      noisedead[1] = "deadconst";
      noisedead[2] = "dead";
      noisedead[3] = "noisyconst";
      noisedead[4] = "noisy";
      char noisevschan[328];
      char noisevsswchan[328];
      char liverpooldata[328];
      
      // Create map between electronics ID & software channel ID
      if (ModuleType[iTELL1]!=0){
        //        cout << " reading in the phi file" << ModuleType[iTELL1] << endl;
        FILE *fm = fopen("/afs/cern.ch/user/l/lhcbvelo/cmtuser/Velo/Vetra/v2r3/macros/pccChannel-mapping.txt", "r");
        for ( i = 0; i < 2048; i++ ) {
          fscanf(fm,"%d%s%d%d%s%d%d%d%d%d",&pad,row,&innerStrip,&outerStrip,pa,&papad,&pachan,&chip,&swstrip,&chipchan);
          swStripList[chipchan] = swstrip;
        }
        fclose(fm);
      } else {
        //        cout << " reading in the r file" << ModuleType[iTELL1] << endl;
        FILE *fm = fopen("/afs/cern.ch/user/l/lhcbvelo/cmtuser/Velo/Vetra/v2r3/macros/rccChannel-mapping.txt", "r");
        for ( i = 0; i < 2048; i++ ) {
          fscanf(fm,"%d%s%d%d%s%d%d%d%d%d",&pad,row,&sector,&strip,pa,&papad,&pachan,&chip,&swstrip,&chipchan);
          swStripList[chipchan] = swstrip;
        }
        fclose(fm);
      }
      
      // Get noise map
      TH1F *hnoiseTotMap = (TH1F *) tdPedAndNoise->Get("hRMSMapHigh");
      TH1F *hnoiseCmsMap = (TH1F *) tdPedAndNoise->Get("hRMSMapHighCMS");
      TH1F *pedestalMap  = (TH1F *) tdPedAndNoise->Get("hPedestalMapHigh");
      
      TH1F *noiseTotMapOld = (TH1F *) hnoiseTotMap->Clone("noiseTotMapOld");
      TH1F *noiseCmsMapOld = (TH1F *) hnoiseCmsMap->Clone("noiseCmsMapOld");
      TH1F *noiseTotMapNew = (TH1F *) hnoiseTotMap->Clone("noiseTotMapNew");
      TH1F *noiseCmsMapNew = (TH1F *) hnoiseCmsMap->Clone("noiseCmsMapNew");
      TH1F *noiseTotMapSfw = (TH1F *) hnoiseTotMap->Clone("noiseTotMapSfw");
      TH1F *noiseCmsMapSfw = (TH1F *) hnoiseCmsMap->Clone("noiseCmsMapSfw");
      TH1F *pedestalSfw    = (TH1F *) pedestalMap ->Clone("pedestalSfw");
      
      noiseTotMapOld->SetTitle("Total noise vs electronics channel");
      noiseCmsMapOld->SetTitle("Cms noise vs electronics channel");
      noiseTotMapNew->SetTitle("Total noise vs electronics channel");
      noiseCmsMapNew->SetTitle("Cms noise vs electronics channel");
      noiseTotMapSfw->SetTitle("Total noise vs software channel");
      noiseCmsMapSfw->SetTitle("Cms noise vs software channel");
      
      TH1F *linkInUse = new TH1F("linkInUse", "Links that are used", 64, -0.5, 63.5);
      TH1F *noiseTotDistAll = new TH1F("noiseTotDistAll", "total noise", 100, 0.0, 5.0);
      TH1F *noiseCmsDistAll = new TH1F("noiseCmsDistAll", "total CM noise", 100, 0.0, 5.0);
      TH1F *noiseTotDistInn = new TH1F("noiseTotDistInn", "total noise of inner P strips", 102, 0.0, 5.0);
      TH1F *noiseTotDistOut = new TH1F("noiseTotDistOut", "total noise of outer P strips", 100, 0.0, 5.0);
      TH1F *noiseCmsDistInn = new TH1F("noiseCmsDistInn", "Cms noise of inner P strips", 102, 0.0, 5.0);
      TH1F *noiseCmsDistOut = new TH1F("noiseCmsDistOut", "Cms noise of outer P strips", 100, 0.0, 5.0);
      

      // Initialization
      for ( iCard = 0; iCard < 4; iCard++ ) {
        for ( iChip = 0; iChip < 4; iChip++ ) {
          for ( iLink = 0; iLink < 4; iLink++ ) {
            linkId = iCard*16 + iChip*4 + iLink;
            linkUsed[linkId]    = 1;
            rawNoise[linkId]    = 0.0;
            sqrawNoise[linkId]  = 0.0;
            cmsNoise[linkId]    = 0.0;
            sqcmsNoise[linkId]  = 0.0;
            rawNoiseRms[linkId] = 0.0;
            cmsNoiseRms[linkId] = 0.0;
          }
        }
      }
      

      // Loop over all repeater cards (sector) and beetle chips and channels
      for ( iCard = 0; iCard < 4; iCard++ ) {
        for ( iChip = 0; iChip < 4; iChip++ ) {
          for ( iChannel = 0; iChannel < 128; iChannel++ ) {
            linkId = iCard*16 + iChip*4 + iChannel/32;
            channelEid = iCard*512 + iChip*128 + iChannel;
            channelSid = swStripList[channelEid];
            
            Float_t noiseTot = noiseTotMapOld->GetBinContent(channelEid+1);
            Float_t noiseCms = noiseCmsMapOld->GetBinContent(channelEid+1);
            
            noiseTotMapNew->SetBinContent(channelEid+1, noiseTot);
            noiseCmsMapNew->SetBinContent(channelEid+1, noiseCms);
            noiseTotMapSfw->SetBinContent(channelSid+1, noiseTot);
            noiseCmsMapSfw->SetBinContent(channelSid+1, noiseCms);
            
            Float_t ped = pedestalMap->GetBinContent(channelEid+1);
            pedestalSfw->SetBinContent(channelSid+1, ped);
            
            //if ( iChannel%32!=0 && iChannel%32!=1) {
            noiseTotDistAll->Fill(noiseTot);
            noiseCmsDistAll->Fill(noiseCms);
            
            rawNoise[linkId]  += noiseTot;
            sqrawNoise[linkId]+= noiseTot*noiseTot;
            cmsNoise[linkId]  += noiseCms;
            sqcmsNoise[linkId]+= noiseCms*noiseCms;
              
            if ( hybridType == "p" ) {
              if ( channelSid < 683 ) {
                noiseTotDistInn->Fill(noiseTot);
                noiseCmsDistInn->Fill(noiseCms);
              } else {
                noiseTotDistOut->Fill(noiseTot);
                noiseCmsDistOut->Fill(noiseCms);
              }
            }
          }
        }
      }

      //loop on links to calculate cuts  
      for ( linkId = 0; linkId <64; linkId++){
        sqrawNoise[linkId] = sqrawNoise[linkId]/32.;
        rawNoise[linkId]   = rawNoise[linkId]/32.;
        sqcmsNoise[linkId] = sqcmsNoise[linkId]/32.;
        cmsNoise[linkId]   = cmsNoise[linkId]/32.;
        rawNoiseRms[linkId]= sqrt(sqrawNoise[linkId]-rawNoise[linkId]*rawNoise[linkId]);
        cmsNoiseRms[linkId]= sqrt(sqcmsNoise[linkId]-cmsNoise[linkId]*cmsNoise[linkId]);
        if ( rawNoise[linkId] < 0.1 ) {
          linkUsed[linkId] = 0;
          if ( printSelect > 0 ) cout << "   -- CERN dead link ID: " << linkId << endl;
        } else {
          linkInUse->Fill(linkId);
        }
      }

      //loop to identify bad channels by applying cut
      int cernBad[16];
      int nBadMatchedTotal = 0;
      int nBadChannelCern  = 0;
      int nBadChannelTotal = 0;
      int nBadChannelNew   = 0;
      for ( iCard = 0; iCard < 4; iCard++ ) {
        for ( iChip = 0; iChip < 4; iChip++ ) {
          chipId = iCard*4 + iChip;
          cernBad[chipId] = 0;
          for ( iChannel = 0; iChannel < 128; iChannel++ ) {
            linkId = iCard*16 + iChip*4 + iChannel/32;

            deadCut  = rawNoise[linkId] - noiseLowLimit;
            noisyCut = cmsNoise[linkId] + noiseUpLimit;
            noisycutused[linkId] = noisyCut;
            deadcutused[linkId]  = deadCut;
            channelEid = iCard*512 + iChip*128 + iChannel;
            channelSid = swStripList[channelEid];
            Float_t noiseTot = noiseTotMapNew->GetBinContent(channelEid+1);
            Float_t noiseCms = noiseCmsMapNew->GetBinContent(channelEid+1);
            Float_t ped      = pedestalMap->GetBinContent(channelEid+1);
            if ( hybridType == "p" && channelSid>683&& (channelSid%2==1)) deadCut=deadCut+0.2;
              
            livBadChannel[channelEid] = 0;
            for (int idummy=0; idummy<nliv; idummy++){
              if (channelSid==ilivList[idummy]){
                livBadChannel[channelEid] = 1;
              }
            }
            
            cernBadChannel[channelEid] = 0; // OK
            if ( linkUsed[linkId] ) {
              if ( noiseTot < noiseLowConst )cernBadChannel[channelEid] = 1;//deadconst
              if ( noiseTot < deadCut )      cernBadChannel[channelEid] = 2;//dead
              if ( noiseCms > noiseUpConst ) cernBadChannel[channelEid] = 3;//noisyconst
              if ( noiseCms > noisyCut )     cernBadChannel[channelEid] = 4;//noisy
            }
            
            if ( cernBadChannel[channelEid] > 0 ) {
              nBadChannelCern ++;
              cernBad[chipId] ++;
              if ( livBadChannel[channelEid] == 0 ) nBadChannelNew++;
            }
            if ( cernBadChannel[channelEid] > 0 && livBadChannel[channelEid] ) {
              nBadMatchedTotal ++;
            }
            if ( livBadChannel[channelEid] || cernBadChannel[channelEid] > 0 ) {
              nBadChannelTotal ++;
            }
          }
        }
      }

      if ( printSelect > 0 ) {
        cout << "  Liverpool bad strips: " << nliv << endl;
        cout << "  CERN found:           " << nBadChannelCern << " bad channels" << endl;
        cout << "     matched:           " << nBadMatchedTotal << endl;
        cout << "         new:           " << nBadChannelNew << endl;
        cout << "  Total bad strips      " << nBadChannelTotal << endl;
      }
      
      if ( printSelect == 2 && nBadChannelNew > 0) {
        cout << "---- List of new found bad strips: " << endl;
        cout << "# chip channel swStrip   totNoise        cmsNoise          status" << endl;
      }
      if ( printSelect == 3 && nBadChannelCern > 0) {
        cout << "---- List of Cern bad strips: " << endl;
        cout << "# chip channel swStrip   totNoise        cmsNoise          status" << endl;
      }
      if ( printSelect == 4 && nliv > 0) {
        cout << "---- List of Liverpool bad strips: " << endl;
        cout << "# chip channel swStrip   totNoise        cmsNoise          status" << endl;
      }
      if ( printSelect == 5 && nBadChannelTotal > 0) {
        cout << "---- List of Cern + Liverpool bad strips: " << endl;
        cout << "# chip channel swStrip   totNoise        cmsNoise          status" << endl;
      }
      
      int nPrint = 0;
      for ( chipId = 0; chipId < 15; chipId++ ) {
        for ( iChannel = 0; iChannel < 128; iChannel++ ) {
          linkId = chipId*4 + iChannel/32;
          channelEid = chipId*128 + iChannel;
          int printOut = 0;
          if ( printSelect==2 && (cernBadChannel[channelEid]>0 && livBadChannel[channelEid]==0) ) printOut=1;
          if ( printSelect==3 && cernBadChannel[channelEid]>0 ) printOut=1;
          if ( printSelect==4 && livBadChannel[channelEid]>0 ) printOut=1;
          if ( printSelect==5 && (cernBadChannel[channelEid]>0 || livBadChannel[channelEid]>0) ) printOut=1;

          if ( printOut && (nPrint < maxPrintOut || maxPrintOut == -1 ) ) {
            channelSid = swStripList[channelEid];          
            Float_t noiseTot = noiseTotMapNew->GetBinContent(channelEid+1);
            Float_t noiseCms = noiseCmsMapNew->GetBinContent(channelEid+1);

            streamsize prec = cout.precision();
            cout.width(4);
            cout << iCard*4+iChip << " ";
            cout.width(7);
            cout << iChannel << " ";
            cout.width(7);
            cout << channelSid << " ";
            cout.precision(3);
            cout.width(7);
            cout << noiseTot << " ";
            cout << "(";
            cout.precision(2);
            cout.width(5);
            cout << noiseTot-rawNoise[linkId] << " ";
            cout << ")";
            cout.precision(3);            
            cout.width(7);
            cout << noiseCms << " ";
            cout << "(";
            cout.precision(2);
            cout.width(5);
            cout << noiseCms-cmsNoise[linkId] << " ";
            cout << ")";
            cout.width(8);
            cout << noisedead[cernBadChannel[channelEid]] << " ";
            cout.precision(prec);
            cout.width(8);
            cout << livnoisedead[livBadChannel[channelEid]] << endl;
            cout.precision(prec);
          }
          if ( printOut ) nPrint ++;
        }
      }

      if ( nPrint > maxPrintOut && maxPrintOut > 0 ) {
        cout << "  ******** " << nPrint-maxPrintOut
             << " more bad channels are not printed out ******" << endl;
      }


      // print plots
      //TCanvas *c1 = new TCanvas("c1","This is what i said tonight",10,10,600,600);
      
      // Links used
      //linkInUse->SetLineColor(4);
      //linkInUse->Draw("hist");
      //c1->Update();
      //c1->Delete();
      //if ( savePlot("Links that are used") ) return;
      
      // Total noise and CM subtracted noise vs electronics channel
      
      if (iTELL1<6) {
        mPad2->cd(iTELL1+1);
      } else {
        mPad3->cd(iTELL1-6+1);
      }
  
      noiseTotMapNew->SetTitle("Noise vs electronic channel");
      noiseTotMapNew->SetMinimum(0.0);
      noiseTotMapNew->GetXaxis()->SetTitle("electronics channel");
      noiseTotMapNew->GetYaxis()->SetTitle("RMS Noise");
      //   noiseTotMapNew->SetMaximum(2.0);
      noiseTotMapNew->SetMaximum(4.5);
      //  noiseTotMapNew->SetMaximum(20);
      noiseTotMapNew->SetLineColor(4);
      noiseTotMapNew->SetLineWidth(1);
      
      noiseCmsMapNew->SetTitle("Noise vs electronic channel");
      noiseCmsMapNew->SetLineColor(2);
      noiseCmsMapNew->SetLineWidth(1);
      noiseTotMapNew->Draw("hist");
      noiseCmsMapNew->Draw("histsame");
      
      sprintf(noisevschan,"%s ",module);
      noiseTotMapNew->SetTitle(noisevschan); 
      for ( linkId = 0; linkId < 64; linkId++ ) {
        TLine *mylinenoise=new TLine(linkId*32,noisycutused[linkId],(linkId+1)*32,noisycutused[linkId]);
        mylinenoise->SetLineColor(6);
        mylinenoise->Draw();
        TLine *mylinedead=new TLine(linkId*32,deadcutused[linkId],(linkId+1)*32,deadcutused[linkId]);
        mylinedead->Draw();
      }
      //      mPad2->Update();
      
      // Total noise and CM subtracted noise vs software channel
      
      if (iTELL1<6) {
        mPad4->cd(iTELL1+1);
      } else{
        mPad5->cd(iTELL1-6+1);
      }
      
      
      noiseTotMapSfw->SetTitle("Noise vs software channel");
      sprintf(noisevschan,"%s ",module);
      //  sprintf(noisevschan,"%s %s side. Noise vs software channel.",title,hybridType);
      
      noiseTotMapSfw->SetTitle(noisevschan); 
      noiseTotMapSfw->GetXaxis()->SetTitle("Software channel");
      noiseTotMapSfw->GetYaxis()->SetTitle("RMS Noise");
      
      noiseTotMapSfw->SetMinimum(0.0);
      // noiseTotMapSfw->SetMaximum(20);
      noiseTotMapSfw->SetMaximum(4.5);
      //    noiseTotMapSfw->SetMaximum(2.0);
      noiseTotMapSfw->SetLineColor(4);
      noiseTotMapSfw->SetLineWidth(1);
      
      noiseCmsMapSfw->SetTitle("Noise vs software channel");
      noiseCmsMapSfw->SetLineColor(2);
      noiseCmsMapSfw->SetLineWidth(1);
      noiseTotMapSfw->Draw("hist");
      noiseCmsMapSfw->Draw("histsame");
  
      // Total noise and CM subtracted noise vs software channel
      //TCanvas *c4 = new TCanvas("c4","In spite of you, it's true",10,10,600,600);
      pedestalMap->SetTitle("Pedestal vs electronic channel");
      pedestalMap->SetMinimum(480.0);
      pedestalMap->SetLineColor(4);
      pedestalMap->SetLineWidth(1);
      //pedestalMap->Draw("hist");
      //c4->Update();
      //if ( savePlot("Pedestal vs electronic channel") ) return;
      
      // Total noise and CM subtracted noise vs software channel
      //TCanvas *c5 = new TCanvas("c5","Scarlet were the clothes she wore,",10,10,600,600);
      pedestalSfw->SetTitle("Pedestal vs software channel");
      pedestalSfw->SetMinimum(480.0);
      pedestalSfw->SetLineColor(4);
      pedestalSfw->SetLineWidth(1);
      //pedestalSfw->Draw("hist");
      //c5->Update();
      //c5->Delete();
      //if ( savePlot("Pedestal vs software channel") ) return;
      

      //TCanvas *c6 = new TCanvas("c6","Everybody knows I'm sure",10,10,600,600);
      // Total noise and CM subtracted noise distribution
      //  noiseTotDistAll->Fit("gaus","");  
      //  noiseCmsDistAll->Fit("gaus","");
      Float_t totMean = noiseTotDistAll->GetMean();
      Float_t totRms  = noiseTotDistAll->GetRMS();
      Float_t cmsMean = noiseCmsDistAll->GetMean();
      Float_t cmsRms  = noiseCmsDistAll->GetRMS();
      cout << "------ Total noise: " << totMean
           << "  ( " << totRms
           << " )    CMs: " << cmsMean
           << "  ( " << cmsRms
           << " ) " << endl;

      Double_t yMaxPlot1 = noiseTotDistAll->GetMaximum(1000000.0);
      Double_t yMaxPlot2 = noiseCmsDistAll->GetMaximum(1000000.0);
      Double_t yMaxPlot = yMaxPlot1 > yMaxPlot2 ? yMaxPlot1 : yMaxPlot2;
      gStyle->SetOptStat(0); // name only
      noiseTotDistAll->SetTitle("Noise distribution");
      noiseTotDistAll->SetMaximum(1.15*yMaxPlot);
      noiseTotDistAll->SetLineColor(4);
      noiseTotDistAll->SetFillColor(4);
      noiseCmsDistAll->SetLineColor(2);
      noiseCmsDistAll->SetLineWidth(2);
      //noiseTotDistAll->Draw("hist");
      //noiseCmsDistAll->Draw("histsame");
      //c6->Update();
      //c6->Delete();
      //if ( savePlot("Noise(total/cms) distribution") ) return;
      
      if ( hybridType == "p" ) {
        //TCanvas *c7 = new TCanvas("c7","",10,10,600,600);
        Double_t yMaxPlot1 = noiseTotDistOut->GetMaximum(1000000.0);
        Double_t yMaxPlot2 = noiseTotDistInn->GetMaximum(1000000.0);
        Double_t yMaxPlot = yMaxPlot1 > yMaxPlot2 ? yMaxPlot1 : yMaxPlot2;
        noiseTotDistOut->SetTitle("Total noise of different strip size");
        noiseTotDistOut->SetMaximum(1.15*yMaxPlot);
        noiseTotDistOut->SetFillColor(4);
        noiseTotDistInn->SetFillColor(2);
        //noiseTotDistOut->Draw("hist");
        //noiseTotDistInn->Draw("histsame");
        //c7->Update();
        //c7->Delete();
        //if ( savePlot("Total noise of inner and outer P strips") ) return;
        
        //TCanvas *c8 = new TCanvas("c8","",10,10,600,600);
        Double_t yMaxPlot1 = noiseCmsDistOut->GetMaximum(1000000.0);
        Double_t yMaxPlot2 = noiseCmsDistInn->GetMaximum(1000000.0);
        Double_t yMaxPlot = yMaxPlot1 > yMaxPlot2 ? yMaxPlot1 : yMaxPlot2;
        noiseCmsDistOut->SetTitle("Cms noise of different strip size");
        noiseCmsDistOut->SetMaximum(1.15*yMaxPlot);
        noiseCmsDistOut->SetFillColor(4);
        noiseCmsDistInn->SetFillColor(2);
        //noiseCmsDistOut->Draw("hist");
        //noiseCmsDistInn->Draw("histsame");
        //c8->Update();
        //c8->Delete();
        //if ( savePlot("Cms noise of inner and outer P strips") ) return;
      }

      //tdADC->cd();
      //hAllChannels->Draw();
      if (iTELL1<6) {
        mPad8->cd(iTELL1+1);
      } else {
        mPad9->cd(iTELL1-6+1);
      }

      TH2F *h2048Channels = (TH2F *)tdADC->Get("hAllChannels");  
      sprintf(noisevschan,"%s ",module);
      h2048Channels->SetTitle(noisevschan); 
      h2048Channels->Draw("box");
      h2048Channels->GetYaxis()->SetRange(450, 1000);
    }
    //    cout << " end of loop over histograms " << endl;
    //    cout << " that loop was for TELL1 number " << iTELL1 << endl;
    delete noiseTotDistAll;
    delete noiseCmsDistAll;
    delete noiseTotDistInn;
    delete noiseTotDistOut;
    delete noiseCmsDistInn;
    delete noiseCmsDistOut; 
    delete linkInUse;
    
 }
  //  cout << " end of loop over tell1s " << endl;

  // Generate one PDF file
  c2->Print(Form("%s.ps(",title));
  c3->Print(Form("%s.ps",title));
  c4->Print(Form("%s.ps",title));
  c5->Print(Form("%s.ps",title));
  c8->Print(Form("%s.ps",title));
  c9->Print(Form("%s.ps)",title));
  gSystem->Exec(Form("ps2pdf %s.ps ; rm -f %s.ps",title,title));
  
}
