// $Id: VeloClusterMoni.h,v 1.3 2006/02/21 17:18:06 szumlat Exp $
#ifndef VELOCLUSTERMONI_H 
#define VELOCLUSTERMONI_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiTupleAlg.h"

// velo
#include "Event/MCVeloFE.h"
#include "VeloEvent/InternalVeloCluster.h"

// from TELL1 event
#include "Event/EvtInfo.h"

/** @class VeloClusterMoni VeloClusterMoni.h
 *  
 *
 *  @author Tomasz Szumlak
 *  @date   2005-11-30
 */

class IMCVeloFEType;
class DeVelo;

class VeloClusterMoni : public GaudiTupleAlg {
public: 
  /// Standard constructor
  VeloClusterMoni( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~VeloClusterMoni( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization
  StatusCode getData();
  StatusCode veloClusterMonitor();
  StatusCode clusterType(LHCb::InternalVeloCluster* clu,
                         bool& s, bool& n, bool& o);
  
protected:

private:

  std::string m_clusterCont;
  std::string m_feCont;
  bool m_printInfo;
  LHCb::InternalVeloClusters* m_veloClusters;
  LHCb::MCVeloFEs* m_veloFEs;
  IMCVeloFEType* m_feTypeTool;
  double m_nVeloClusters;
  double m_nVeloClusters2;
  double m_nVeloClustersS;
  double m_nVeloClustersN;
  double m_nVeloClustersO;
  double m_nOneStrip;
  double m_nTwoStrip;
  double m_nThreeStrip;
  double m_nFourStrip;
  int m_numberOfEvents;
  DeVelo* m_veloDet;
  signed int m_convergeLimit; /// A stand-in for the convergence limit, which is difficult to access from here
  signed int m_headerWidth; // Another option - how many header strips to mask

  // hit tree variables, giving data about each cluster
  signed int m_firstPosX; // Position of first strip in cluster
  unsigned int m_nStrips; // No of strips in cluster
  signed int m_centrePosX; // strip no of strip with largest cluster
  signed int m_centreADC; // ADC value of strip with biggest signal
  signed int m_clusterADC; // Total signal in cluster

  // Variables for writing event trees
  // no of hits on each detector
  unsigned int m_numberOfHitsGla0;
  unsigned int m_numberOfHitsGla1;
  unsigned int m_numberOfHitsGla2;
  unsigned int m_numberOfHitsFre0;
  unsigned int m_numberOfHitsFre1;
  unsigned int m_numberOfHitsFre2;

    // Extra event information
  unsigned int m_bunchCounter;
  unsigned long int m_BCntLoop;  /// In this version, this variable both fixes the problems caused by the bunch counter looping back to zero, and also increases with the BATTrig value. So, by sorting the tree by this variable we can get all the "events" from the TELL1 data file in the right order, ready for analysis.
  bool m_doesCounterLoop;

  unsigned int m_BATTrig;
  unsigned int m_orderRaw;

  EvtInfos* m_evtInfos;              /// container for Event Info blocks
  std::string m_evtInfoLoc;          /// location of decoded Event Info

};
#endif // VELOCLUSTERMONI_H
