// $Id: ClusterDataMoniUserHistos.cpp,v 1.10 2007/08/27 12:54:15 szumlat Exp $

// Include files 


// from Gaudi
#include "GaudiKernel/AlgFactory.h"
#include "GaudiKernel/SystemOfUnits.h"

// velo

#include "VeloDet/DeVelo.h"
#include "VeloDet/DeVeloSensor.h"

#include "Kernel/VeloChannelID.h"
#include "Event/VeloCluster.h"

//boost
#include <boost/lexical_cast.hpp>

// system
#include <math.h>

// local
#include "ClusterDataMoniUserHistos.h"

//#include "AIDA/IHistogram1D.h"

using namespace Gaudi::Units;

//-----------------------------------------------------------------------------
// Implementation file for class : ClusterDataMoniUserHistos
//
// 2006-07-20:   Aras Papadelis, Thijs Versloot
//
//
//-----------------------------------------------------------------------------

// Declaration of the Algorithm Factory
DECLARE_ALGORITHM_FACTORY( ClusterDataMoniUserHistos );


//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
ClusterDataMoniUserHistos::ClusterDataMoniUserHistos( const std::string& name,
						      ISvcLocator* pSvcLocator)
  : VeloClusterDataMonitor ( name , pSvcLocator ),
    m_veloDet ( 0 ),
    m_numberOfEvents ( 0 )
    
{  
  declareProperty( "PlotHistos"	, m_plotHistos = false);
  x_mod1.reserve(10000);
  y_mod1.reserve(10000);
  x_mod2.reserve(10000);
  y_mod2.reserve(10000);
  x_mod3.reserve(10000);
  y_mod3.reserve(10000);
}
//=============================================================================
// Destructor
//=============================================================================
ClusterDataMoniUserHistos::~ClusterDataMoniUserHistos() {} 

//=============================================================================
// Initialization
//=============================================================================
StatusCode ClusterDataMoniUserHistos::initialize() {
  StatusCode sc = GaudiTupleAlg::initialize(); // must be executed first
  if ( sc.isFailure() ) return sc;  // error printed already by VeloClusterDataMonitor

  debug() << "==> Initialize" << endmsg;

  m_veloDet=DetectorElement();
  
  return StatusCode::SUCCESS;
}

//=============================================================================
// Main execution
//=============================================================================
StatusCode ClusterDataMoniUserHistos::execute() {

  debug() << "==> Execute" << endmsg;

  m_numberOfEvents++;
  

  StatusCode sc=getClusterContainer();
  if(!sc)
    return StatusCode::SUCCESS;
  
  m_clusters=ClusterContainer();
  
  if(m_plotHistos) userHistos();
     
  
  
  return StatusCode::SUCCESS;
}

//=============================================================================
//  Finalize
//=============================================================================
StatusCode ClusterDataMoniUserHistos::finalize() {

  debug() << "==> Finalize" << endmsg;
  
  return GaudiTupleAlg::finalize();  // must be called after all other actions
}



//=============================================================================
// A good place to add new histograms:
//=============================================================================
StatusCode ClusterDataMoniUserHistos::userHistos(){
 
  std::string histoTitle;
  std::string histoName;

  LHCb::VeloClusters::iterator it;
  double channelID1[100000];
  double channelID3[100000];
  double channelID5[100000];
  double channelID65[100000];
  double channelID67[100000];
  double channelID69[100000];


  //  double radius1=0;
  // double radius2=0;

  int sensor;

  int c=0;
  int a=0;
  int b=0;
  int d=0;
  int e=0;
  int f=0;

  int count_r=0;
  int count_phi=0;
  double n_channel=0;

  //Begin loop over clusters:
  for (it=m_clusters->begin(); it!=m_clusters->end(); it++) {
    
    LHCb::VeloCluster* cluster = (*it);
    cluster=cluster; //this is just to get rid of warning messages at compile time. 
    
    plot1D(cluster->totalCharge(),"ExamplePlot", "This is an example plot",
           0,512,512);

    //------------------------------------------------------------------------------------------------------------------------------------------------------
    LHCb::VeloChannelID idtemp = cluster->channelID();

    int clsns = idtemp.sensor();
    const DeVeloSensor* temps = m_veloDet->sensor(clsns);
    double z_pos = temps->z();
    //int clstr = idtemp.strip();
    unsigned int clsize = cluster->size();

    //    double strfr = cluster->interStripFraction();

    plot1D(z_pos, "nClustervsZ", "Number of Clusters vs z(cm)", -300.0, 200.0,100);

    //------------------------------------------------------------------------------------------------
    //Constructing arrays of R sensors channels
    //-----------------------------------------------------------------------------------------------


    if(temps->isR()){



      for(unsigned int istrip=0;istrip<clsize; istrip++){

	sensor = clsns;
	n_channel = (cluster->stripValues())[istrip].first;


	//if(!isnan(n_channel)){
  if(n_channel<200000){
    
	  if(sensor==7){
	    channelID1[a] = (cluster->stripValues())[istrip].first;
	    plot1D(channelID1[a],"channelID_s7","Channel ID of clusters in sensor 7",-0.5,2048,2048);
	    count_r++;
	    a++;
	  }

	  if(sensor==3){
	    channelID3[b] = (cluster->stripValues())[istrip].first;
	    plot1D(channelID3[b],"channelID_s3","Channel ID of clusters in sensor 3",-0.5,2048,2048);
	    count_r++;
	    b++;
	  }

	  if(sensor==5){
	    channelID5[c] = (cluster->stripValues())[istrip].first;
	    plot1D(channelID5[c],"channelID_s5","Channel ID of clusters in sensor 5",-0.5,2048,2048);

	    count_r++;
	    c++;
	  }
	}
      }

    }

    //------------------------------------------------------------------------------------------------------------------
    //Constructing arrays of Phi sensors channels
    //-----------------------------------------------------------------------------------------------------------------

    if(temps->isPhi()){


      for(unsigned int istrip=0;istrip<clsize; istrip++){

	sensor = clsns;

	//if(!isnan(n_channel)){
  if(n_channel<200000){

	  if(sensor==71){
	    channelID65[d] = (cluster->stripValues())[istrip].first;
	    plot1D(channelID65[d],"channelID_s71","Channel ID of clusters in sensor 71",-0.5,2048,2048);

	    count_phi++;
	    d++;
	  }

	  if(sensor==67){
	    channelID67[e] = (cluster->stripValues())[istrip].first;
	    plot1D(channelID67[e],"channelID_s67","Channel ID of clusters in sensor 67",-0.5,2048,2048);

	    count_phi++;
	    e++;
	  }

	  if(sensor==69){
	    channelID69[f] = (cluster->stripValues())[istrip].first;

	    plot1D(channelID69[f],"channelID_s69","Channel ID of clusters in sensor 69",-0.5,2048,2048);
	    count_phi++;
	    f++;
	  }
	}
      }
    }

    
    
  }

    bool profile = false;  

    //-----------------------------------------------------------------------------------------------------------
    //Plotting R sensors pair| Channels vs Channels
    //----------------------------------------------------------------------------------------------------------
 
    for(a=0; a<count_r; a++){

      if(profile==true){
   
	profile1D(channelID1[a],channelID3[a],"channel_s7_vs_channel_s3", "Channels of R sensor7 vs Channels of R sensor3, All event", -0.5, 2047.0, 512, 1.0);
	profile1D(channelID1[a],channelID5[a],"channel_s7_vs_channel_s5", "Channels of R sensor7 vs Channels of R sensor5, All event", -0.5, 2047.0, 512, 1.0);
	profile1D(channelID3[a],channelID5[a],"channel_s3_vs_channel_s5", "Channels of R sensor3 vs Channels of R sensor5, All event", -0.5, 2047.0, 512, 1.0);
      }
      else{
	plot2D(channelID1[a],channelID3[a],"channel_s7_vs_channel_s3", "Channels of R sensor7 vs Channels of R sensor3, All event", -0.5, 2047.0, -0.5, 2047, 512, 512);
	plot2D(channelID1[a],channelID5[a],"channel_s7_vs_channel_s5", "Channels of R sensor7 vs Channels of R sensor5, All event", -0.5, 2047.0, -0.5, 2047, 512, 512);
	plot2D(channelID3[a],channelID5[a],"channel_s3_vs_channel_s5", "Channels of R sensor3 vs Channels of R sensor5, All event", -0.5, 2047.0, -0.5, 2047, 512, 512);
      }
    }

    //----------------------------------------------------------------------------------------------------
    //Plotting Phi sensors pair| Channels vs Channels
    //---------------------------------------------------------------------------------------------------


    for(a=0; a<count_phi; a++){

      if (profile==true){
 
	profile1D(channelID65[a],channelID67[a],"channel_s71_vs_channel_s67", "Channels of Phi sensor71 vs Channels of Phi sensor67, All event", -0.5, 2047.0, 512, 1.0);
	profile1D(channelID65[a],channelID69[a],"channel_s71_vs_channel_s69", "Channels of Phi sensor71 vs Channels of Phi sensor69, All event", -0.5, 2047.0, 512, 1.0);
	profile1D(channelID67[a],channelID69[a],"channel_s67_vs_channel_s69", "Channels of Phi sensor67 vs Channels of Phi sensor69, All event", -0.5, 2047.0, 512, 1.0);
      }
      else{
    
	plot2D(channelID65[a],channelID67[a],"channel_s71_vs_channel_s67", "Channels of Phi sensor71 vs Channels of Phi sensor67, All event", -0.5, 2047.0, -0.5, 2047, 512, 512);
	plot2D(channelID65[a],channelID69[a],"channel_s71_vs_channel_s69", "Channels of Phi sensor71 vs Channels of Phi sensor69, All event", -0.5, 2047.0, -0.5, 2047, 512, 512);
	plot2D(channelID67[a],channelID69[a],"channel_s67_vs_channel_s69", "Channels of Phi sensor67 vs Channels of Phi sensor69, All event", -0.5, 2047.0, -0.5, 2047, 512, 512);
      }		    
    }


    //    for(a=0; a<count; a++){

    //      plot2D(x_pos_mod1[a], x_pos_mod2[a], "x_mod1_vs_x_mod2","X positions of module 1 vs x positions of module 2", -40
 
    //    return StatusCode::SUCCESS;

    //  }


    return StatusCode::SUCCESS;
}



