// $Id: VeloClusterDataMonitor.h,v 1.9 2007/07/06 13:42:06 arasp Exp $

#ifndef VELOCLUSTERDATAMONITOR_H 
#define VELOCLUSTERDATAMONITOR_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiTupleAlg.h"

/** @class VeloClusterDataMonitor VeloClusterDataMonitor.h
 *  
 *
 *  @author Aras Papadelis, Thijs Versloot
 *  @date   2006-04-30

*/

class DeVelo;


class VeloClusterDataMonitor : public GaudiTupleAlg {
public: 
  /// Standard constructor
  VeloClusterDataMonitor( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~VeloClusterDataMonitor( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization


  LHCb::VeloClusters* ClusterContainer()
  {
    return m_clusters;    
  }
  
  DeVelo* DetectorElement() 
  {
    return m_veloDet;
    
  }
  
  
  StatusCode getClusterContainer();
  
private:

protected:

  StatusCode clusterMonitor();

  StatusCode clusterNtupleMonitor();

  
  const bool VeloClusterDataMonitor::sensorIsReadOut(int sensorN) const;

  StatusCode hitmapHistos(const DeVeloSensor* temps, LHCb::VeloCluster* cluster);

  StatusCode drawSensorContour();
  

  StatusCode histogramBooking();
  
  DeVelo* m_veloDet;
  
  std::string m_clusterCont;

  int  m_numberOfEvents;

  bool m_printInfo;  
  bool m_clusterInfo;
  bool m_ntuple;
  bool m_sensorOcc;
  bool m_sensorMul;
  bool m_hitmapPlots;

  double m_hitmapres;
  int m_hitmapadccut;
  
  
  double m_xrange, m_yrange;
  int m_xbins,m_ybins;
  
  LHCb::VeloClusters* m_clusters;
  
  std::map<int, double> m_rcluster;
  std::map<int, double> m_phicluster;
  std::vector<double> m_nClustersPerSensor;

  
  double m_nVC;    	// Number of VeloClusters
  int m_nEvent;         // Number of Events

  int m_nRMeasEvent;	// Number of R Measurements per Event
  int m_nPhiMeasEvent;  // Number of Phi Measurements per Event
  int m_nMisMatch;
  
  
  //ntuple variables:
  
  const int m_ntupleSize;
  
  
  std::vector<double> m_clusterCentrePosR;
  std::vector<int> m_clusterTotalChargeR;
  std::vector<int>    m_clusterSizeR;
  std::vector<double> m_clusterGeomPosR;
  std::vector<double> m_clusterZR;
  std::vector<int>    m_clusterSensorIDR;
  

  
  std::vector<double> m_clusterCentrePosPhi;
  std::vector<int> m_clusterTotalChargePhi;
  std::vector<int>    m_clusterSizePhi;
  std::vector<double> m_clusterGeomPosPhi;
  std::vector<double> m_clusterZPhi;
  std::vector<int>    m_clusterSensorIDPhi;
  
  
};

#endif // VELOCLUSTERDATAMONITOR_H
