// $Id: $
#ifndef VELOCLUSTERIZATIONADAPTOR_H 
#define VELOCLUSTERIZATIONADAPTOR_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiAlgorithm.h"

// form TELL1
#include "Event/VeloTELL1Data.h"

// digi event
#include "Event/VeloDigit.h"

// kernel event
#include "Kernel/VeloChannelID.h"

/** @class VeloClusterizationAdaptor VeloClusterizationAdaptor.h
 *  
 *  This algorithm is meant as mean to translate between
 *  two different Velo data interfaces - VeloDigit used in
 *  official LHCb simulation and VeloTELL1Data that is used
 *  widely in real data prcessing/emulating. This algorithm
 *  takes VeloTELL1Data, puts an initial cut on ADC value
 *  (by the ADC we mean digitized charge collected on a strip)
 *  and creates a collection of VeloDigit objects that can
 *  be subsequently passed into VeloClusterMaker algorithm.
 *
 *  @author Tomasz Szumlak
 *  @date   2007-11-25
 */

class DeVelo;

class VeloClusterizationAdaptor : public GaudiAlgorithm {
public: 
  /// Standard constructor
  VeloClusterizationAdaptor( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~VeloClusterizationAdaptor( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:

  StatusCode getData();
  StatusCode translateData();
  StatusCode storeDigiData();

private:

  DeVelo* m_velo;
  std::string m_inDataLoc;
  LHCb::VeloTELL1Datas* m_inData;
  std::string m_outDataLoc;
  LHCb::VeloDigits* m_outData;
  unsigned int m_sensorNumber;
  bool m_useDummySensor;
  bool m_dataWithDummy;
  signed int m_adcThreshold;
  
};
#endif // VELOCLUSTERIZATIONADAPTOR_H
