// $Id: VeloTELL1Algorithm.cpp,v 1.5 2007/08/26 16:39:14 szumlat Exp $
// Include files 

// from Gaudi
#include "GaudiKernel/DeclareFactoryEntries.h" 

// local
#include "VeloTELL1Algorithm.h"

// standard lib
#include <iostream>
#include <fstream>

using namespace LHCb;

//-----------------------------------------------------------------------------
// Implementation file for class : VeloTELL1Algorithm
//
// 2006-12-06 : Tomasz Szumlak
//-----------------------------------------------------------------------------

// Declaration of the Algorithm Factory
DECLARE_ALGORITHM_FACTORY( VeloTELL1Algorithm );

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
VeloTELL1Algorithm::VeloTELL1Algorithm( const std::string& name,
                                        ISvcLocator* pSvcLocator)
  : GaudiAlgorithm ( name , pSvcLocator ),
    m_evtNumber ( 0 ),
    m_validationRun ( false ),
    m_inputDataLoc ( ),
    m_outputDataLoc ( ),
    m_algorithmName ( ),
    m_detElement ( 0 ),
    m_inputData ( 0 ),
    m_outputData ( 0 ),
    m_procInfos ( 0 ),
    m_procInfosLoc ( VeloProcessInfoLocation::Default ),
    m_algorithmType ( ),
    m_isEnable ( false ),
    m_isInitialized ( false )
{

}
//=============================================================================
// Destructor
//=============================================================================
VeloTELL1Algorithm::~VeloTELL1Algorithm() {} 

//=============================================================================
// Initialization
//=============================================================================
StatusCode VeloTELL1Algorithm::initialize() {
  StatusCode sc = GaudiAlgorithm::initialize(); // must be executed first
  if ( sc.isFailure() ) return sc;  // error printed already by GaudiAlgorithm

  debug() << "==> Initialize" << endmsg;
  m_detElement=getDet<DeVelo>(DeVeloLocation::Default);
  return StatusCode::SUCCESS;
}

//=============================================================================
// Main execution
//=============================================================================
StatusCode VeloTELL1Algorithm::execute() {

  debug() << "==> Execute" << endmsg;

  return StatusCode::SUCCESS;
}

//=============================================================================
//  Finalize
//=============================================================================
StatusCode VeloTELL1Algorithm::finalize() {

  debug() << "==> Finalize" << endmsg;

  return GaudiAlgorithm::finalize();  // must be called after all other actions
}
//=============================================================================
StatusCode VeloTELL1Algorithm::getData()
{
  debug()<< " ==> getData() " <<endmsg;
  //
  if(!exist<LHCb::VeloTELL1Datas>(m_inputDataLoc)){
    debug()<< " ==> There is no input data: "
           << m_inputDataLoc <<endmsg;
    return ( StatusCode::FAILURE );
  }else{  
    // get data banks from default TES location
    m_inputData=get<LHCb::VeloTELL1Datas>(m_inputDataLoc);
    debug()<< " ==> The data have been read-in from location: "
           << m_inputDataLoc
           << ", size of the data container: "
           << m_inputData->size() <<endmsg;
    // vector for reordered data
    if(!m_outputDataLoc.empty()){
      m_outputData=new LHCb::VeloTELL1Datas();
    }
  }
  //
  return ( StatusCode::SUCCESS );
}
//=============================================================================
StatusCode VeloTELL1Algorithm::INIT()
{
  debug()<< " ==> INIT() " <<endmsg;
  //
  if(!m_isInitialized){
    if(!exist<VeloProcessInfos>(m_procInfosLoc)){
      debug()<< " ==> No init info available! " <<endmsg;
    }else{
      m_procInfos=get<VeloProcessInfos>(m_procInfosLoc);
      m_isInitialized=true;
    }
    VeloProcessInfos::const_iterator procIt=m_procInfos->begin();
    bool bit=false;
    if(m_tell1Process!=CLUSTER_MAKER){
      bit=(*procIt)->isEnable(m_tell1Process);
    }else if(m_tell1Process==CLUSTER_MAKER){
      bit=true;
    }
    //
    if((*procIt)->convLimit()<=0){
      Error("Convergence Limit not set!");
      return ( StatusCode::FAILURE );
    }else{
      m_convergenceLimit=(*procIt)->convLimit();
    }
    m_runType=(*procIt)->runType();
    //
    if(bit){
      m_isEnable=true;
      info()<< " ==> " << m_algorithmName << " is enabled! ";
      if(!m_algorithmType.empty()){
        info()<< " type: " << m_algorithmType <<endmsg;
      }else{
        info()<<endmsg;
      }
      return ( StatusCode::SUCCESS );
    }else{
      m_isEnable=false;
      info()<< " ==> " << m_algorithmName << " is disabled! " <<endmsg;
      return ( StatusCode::FAILURE );
    }
  }else{
    return ( StatusCode::SUCCESS );
  }
}
//=============================================================================
StatusCode VeloTELL1Algorithm::writeData()
{
  debug()<< " ==> writeData() " <<endmsg;
  //
  if(m_outputData!=0){
    put(m_outputData, m_outputDataLoc);
    return ( StatusCode::SUCCESS );
  }else{
    error()<< " ==> empty output container! " <<endmsg;
    return ( StatusCode::FAILURE );
  }
}
//=============================================================================
StatusCode VeloTELL1Algorithm::cloneData()
{
  debug()<< " ==> cloneData() " <<endmsg;
  // if CMS algorithm is disabled copy the data for next algo
  VeloTELL1Datas::const_iterator sensIt=m_inputData->begin();
  for( ; sensIt!=m_inputData->end(); ++sensIt){
    const int sens=(*sensIt)->key();
    LHCb::VeloTELL1Data* out=new LHCb::VeloTELL1Data(sens, VeloFull);
    sdataVec replica=(*sensIt)->data();
    out->setDecodedData(replica);
    m_outputData->insert(out);
  }
  StatusCode writer=writeData();
  return ( writer );
}
//=============================================================================
LHCb::VeloTELL1Datas* VeloTELL1Algorithm::inputData() const
{
  debug()<< " ==> inputData() " <<endmsg;
  //
  return ( m_inputData );
}
//=============================================================================
void VeloTELL1Algorithm::outputData(LHCb::VeloTELL1Data* inData)
{
  debug()<< " ==> outputData() " <<endmsg;
  //
  m_outputData->insert(inData);
}
//=============================================================================
LHCb::VeloTELL1Datas* VeloTELL1Algorithm::outputData()
{
  return ( m_outputData );
}
//=============================================================================
void VeloTELL1Algorithm::setInputDataLoc(const std::string inValue)
{
  m_inputDataLoc=inValue;
}
//=============================================================================
void VeloTELL1Algorithm::setOutputDataLoc(const std::string inValue)
{
  m_outputDataLoc=inValue;
}
//=============================================================================
void VeloTELL1Algorithm::setTELL1Process(const unsigned int inValue)
{
  m_tell1Process=inValue;
}
//=============================================================================
void VeloTELL1Algorithm::setAlgoName(const std::string inValue)
{
  m_algorithmName=inValue;
}
//=============================================================================
void VeloTELL1Algorithm::setAlgoType(const std::string inValue)
{
  m_algorithmType=inValue;
}
//=============================================================================
bool VeloTELL1Algorithm::isEnable() const
{
  return ( m_isEnable );
}
//=============================================================================
unsigned int VeloTELL1Algorithm::convergenceLimit() const
{
  return ( m_convergenceLimit );
}
//=============================================================================
bool VeloTELL1Algorithm::isInitialized() const
{
  return ( m_isInitialized );
}
//=============================================================================
unsigned int VeloTELL1Algorithm::runType() const
{
  return ( m_runType );
}
//=============================================================================
StatusCode VeloTELL1Algorithm::readDataFromFile(
                             const char* fileName, sdataVec& dataVec)
{
  debug()<< " readDataFromFile() " <<endmsg;
  //
  sdataVec localStorage;
  std::ifstream inData(fileName);
  signed int aValue=0;
  if(!inData){
    Error(" Can't open the specified file! ");
    info()<< " ===> " << fileName << " <=== " <<endmsg;
    Error(" Mind the input and output file paths ");
    return ( StatusCode::FAILURE );
  }else{
    info()<< " Reading the file " << fileName <<endmsg;
    for(int chan=0; chan<ALL_STRIPS; ++chan){
      inData>>aValue;
      localStorage.push_back(aValue);
    }
    inData.close();
  }
  //
  dataVec=localStorage;
  return ( StatusCode::SUCCESS );
}
//=============================================================================
StatusCode VeloTELL1Algorithm::writeDataToFile(
                             const char* fileName, sdataVec& dataVec)
{
  debug()<< " writeDataToFile() " <<endmsg;
  //
  std::ofstream outData(fileName);
  sdatIt datIT=dataVec.begin();
  if(outData.bad()){
    Error(" Cannot open file to write ");
    return ( StatusCode::FAILURE );
  }else{
    for( ; datIT!=dataVec.end(); ++datIT){
      outData<<(*datIT)<<std::endl;
    }
    outData.close();
  }
  //
  return ( StatusCode::SUCCESS );
}
//=============================================================================
void VeloTELL1Algorithm::prepareData(bool insertAtEnd=false) const
{
  debug()<< " ==> prepareData() " <<endmsg;
  //
  LHCb::VeloTELL1Datas::const_iterator sensIt;
  sensIt=inputData()->begin();
  sdataVec data, dummy(64, 0), dummy1(256, 512);
  sdatIt dumIt;
  for( ; sensIt!=inputData()->end(); ++sensIt){
    data.clear();
    data=(*sensIt)->data();
    if(!insertAtEnd){
      unsigned int proc=0, index=0;
      while(proc<NumberOfPPFPGA){
        index=(proc+1)*STRIPS_IN_PPFPGA+proc*DUMMY_STRIPS;
        std::insert_iterator<sdataVec> ii(data, (data.begin()+index));
        for(dumIt=dummy.begin(); dumIt!=dummy.end(); ++dumIt){
          *ii++=(*dumIt);
        }
        ++proc;
      } 
    }else{
      std::insert_iterator<sdataVec> jj(data, data.end());
      for(dumIt=dummy1.begin(); dumIt!=dummy1.end(); ++dumIt){
        *jj++=(*dumIt);
      }
    }
    // check if there is propre number of strips
    assert(data.size()==ALL_STRIPS);
    (*sensIt)->setDecodedData(data);
  }
}
//=============================================================================
void VeloTELL1Algorithm::prepareData(sdataVec& inVec) const
{
  debug()<< " ==> prepareData(sdataVec) " <<endmsg;
  //
  sdataVec data, dummy(64, 0);
  sdatIt dumIt;
  unsigned int proc=0, index=0;
  while(proc<NumberOfPPFPGA){
    index=(proc+1)*STRIPS_IN_PPFPGA+proc*DUMMY_STRIPS;
    std::insert_iterator<sdataVec> ii(inVec, (inVec.begin()+index));
    for(dumIt=dummy.begin(); dumIt!=dummy.end(); ++dumIt){
      *ii++=(*dumIt);
    }
    ++proc;
  }
}
//=============================================================================
DeVelo* VeloTELL1Algorithm::deVelo() const
{
  debug()<< " ==> deVelo() "<<endmsg;
  return ( m_detElement );
}
//=============================================================================
StatusCode VeloTELL1Algorithm::inputStream(VeloTELL1Datas* inVec) const
{
  debug()<< " ==> dataStatus() " <<endmsg;
  if(inVec->empty()){
    Error("Input stream is empty!");
    return ( StatusCode::FAILURE );
  }else{
    return ( StatusCode::SUCCESS);
  }
}
//--
