// $Id: VeloTELL1Algorithm.h,v 1.5 2007/08/26 16:39:14 szumlat Exp $
#ifndef VELOTELL1ALGORITHM_H 
#define VELOTELL1ALGORITHM_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiAlgorithm.h"

// form TELL1 Kernel
#include "Kernel/DecodeCore.h"
#include "Kernel/DecodeConf.h"
#include "Kernel/TELL1Core.h"

// from TELL1 event
#include "Event/VeloTELL1Data.h"
#include "Event/VeloProcessInfo.h"

// VeloDet
#include "VeloDet/DeVelo.h"

/** @class VeloTELL1Algorithm VeloTELL1Algorithm.h
 *  
 *  base class for any TELL1 algorithm
 *
 *  @author Tomasz Szumlak
 *  @date   2006-12-06
 */

using namespace VeloTELL1;

class VeloTELL1Algorithm : public GaudiAlgorithm {
public: 
  /// Standard constructor
  VeloTELL1Algorithm( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~VeloTELL1Algorithm( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:
  
  virtual StatusCode getData();
  virtual StatusCode INIT();
  virtual StatusCode writeData();
  virtual StatusCode cloneData();
  virtual void setInputDataLoc(const std::string inValue);
  virtual void setOutputDataLoc(const std::string inValue);
  virtual void setTELL1Process(const unsigned int inValue);
  virtual void setAlgoName(const std::string inValue);
  virtual void setAlgoType(const std::string inValue);
  virtual LHCb::VeloTELL1Datas* inputData() const;
  virtual void outputData(LHCb::VeloTELL1Data* inData);
  virtual LHCb::VeloTELL1Datas* outputData();
  virtual bool isEnable() const;
  virtual unsigned int convergenceLimit() const;
  virtual unsigned int runType() const;
  virtual bool isInitialized() const;
  virtual StatusCode readDataFromFile(const char* fileName, sdataVec& inVec);
  virtual StatusCode writeDataToFile(const char* fileName, sdataVec& inVec);
  virtual StatusCode inputStream(LHCb::VeloTELL1Datas* inVec) const;
  unsigned int m_evtNumber;
  bool m_validationRun;
  std::string m_inputDataLoc;
  std::string m_outputDataLoc;
  std::string m_algorithmName;
  virtual void prepareData(bool insertAtEnd) const;
  virtual void prepareData(sdataVec& inVec) const;
  virtual DeVelo* deVelo() const;

private:

  DeVelo* m_detElement;
  LHCb::VeloTELL1Datas* m_inputData;
  LHCb::VeloTELL1Datas* m_outputData;
  VeloProcessInfos* m_procInfos;
  std::string m_procInfosLoc;
  std::string m_algorithmType;
  unsigned int m_tell1Process;
  bool m_isEnable;
  unsigned int m_convergenceLimit;
  bool m_isInitialized;
  unsigned int m_runType;

};
#endif // VELOTELL1ALGORITHM_H
