// $Id: VeloTELL1ClusterMaker.h,v 1.6 2007/08/26 16:39:14 szumlat Exp $
#ifndef VELOTELL1CLUSTERMAKER_H 
#define VELOTELL1CLUSTERMAKER_H 1

// Include files
#include "VeloTELL1Algorithm.h"

// event
#include "Event/VeloProcessInfo.h"
#include "Event/RawEvent.h"

// Si
#include "SiDAQ/SiHeaderWord.h"
#include "SiDAQ/SiRawBufferWord.h"

// local
#include "ClusterMakerThresholds.h"

/** @class VeloTELL1ClusterMaker VeloTELL1ClusterMaker.h
 *  
 *  The algorithm is an implementation of the tell1 cluster
 *  maker that is a part of the tell1lib library. The input 
 *  of the algorithm is the data after reordering and the output
 *  is the RawEvent structure with encoded clusters.
 *  Values of the thresholds are defined inside ClusterMakerThresholds.h
 *  the settings are most critical for the algorithm since
 *  the values are used to serch for seeding strips.
 *  At the end of the algorithm RawEvent structure with RawBanks
 *  containing zero suppressed data is created. The RawEvent is
 *  subsequently stored in TES and decoded to VeloLite and VeloClusters
 *  by VeloDAQ package.
 *
 *  @author Tomasz Szumlak
 *  @date   2006-10-06
 */

//using namespace LHCb;

using namespace VeloTELL1;
using namespace LHCb;

class TELL1ZSProcessEngine;

class VeloTELL1ClusterMaker : public VeloTELL1Algorithm{

public:

  enum version{
    V2=2
  };

  enum positionConst{
    POS_SHIFT=3,
    IS_POS_MASK=0x7,     /// inter strip position mask
    POS_MASK=0x7FF,
    BIT12_MAX=0x1000,
    BIT13_MAX=8192,
    BIT16_MAX=65536
  };

  /// Standard constructor
  VeloTELL1ClusterMaker(const std::string& name, ISvcLocator* pSvcLocator);

  virtual ~VeloTELL1ClusterMaker( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:

  StatusCode runClusterMaker();
  StatusCode veloDataSuppression();
  void calculateClusterPosition(int chan, int cluSize, int firstStripInBlock);
  void addADCToList(int chan, int cluSize);
  StatusCode veloClusterization(int firstStripInBlock);
  void flushMemory();
  StatusCode fillAndWriteRawEvent();
  StatusCode storeBank();
  StatusCode createRawEvent();
  StatusCode validationRun();
  void prepareEngine();

private:

  RawEvent* m_rawEvent;
  std::string m_rawEventLoc;
  u_int8_t m_hitThresholdValue;
  u_int32_t m_lowThresholdValue;
  u_int32_t m_sumThresholdValue;
  HitThresholdVec m_hitThresholds;
  Thresholds m_lowThresholds;
  Thresholds m_sumThresholds;
  TELL1ClusterVec m_clusters;
  CLUSTER_MEMORY m_clustersMem;  /// list of created clusters
  TELL1ADCVec m_adcs;
  ADC_MEMORY m_adcsMem;          /// list of adcs contributed to the clusters
  BoundaryStripVec m_boundaryStripsR;
  BoundaryStripVec m_boundaryStripsPhi;
  unsigned int m_eventNumber;
  dataVec m_sensors;                 /// container with source numbers
  std::vector<SiDAQ::buffer_word> m_bankBody;
  const int m_bankVersion;
  unsigned int m_bankBodySize;
  bool m_printInfo;
  bool m_validationRun;
  TELL1ZSProcessEngine* m_zsEngine;

};
#endif // VELOTELL1CLUSTERMAKER_H
