// $Id: VeloTELL1EmulatorInit.cpp,v 1.6 2007/08/26 16:39:14 szumlat Exp $
// Include files 

// from Gaudi
#include "GaudiKernel/DeclareFactoryEntries.h" 

// local
#include "VeloTELL1EmulatorInit.h"

// STL
#include <vector>
#include <iterator>

//-----------------------------------------------------------------------------
// Implementation file for class : VeloTELL1EmulatorInit
//
// 2006-07-10 : Tomasz Szumlak
//-----------------------------------------------------------------------------

// Declaration of the Algorithm Factory
DECLARE_ALGORITHM_FACTORY( VeloTELL1EmulatorInit );


//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
VeloTELL1EmulatorInit::VeloTELL1EmulatorInit( const std::string& name,
                                              ISvcLocator* pSvcLocator)
  : GaudiAlgorithm ( name , pSvcLocator ),
    m_evtInfos ( 0 ),
    m_evtInfoLoc ( EvtInfoLocation::Default ),
    m_data ( 0 ),
    m_dataLoc ( LHCb::VeloTELL1DataLocation::ADCs ),
    m_mode ( 99 ),
    m_initInfo ( 0 ),
    m_initInfoLoc ( VeloProcessInfoLocation::Default ),
    m_simProc ( 7, 0 )
{
  declareProperty("SimProcessing", m_simProc);
  declareProperty("PrintInfo", m_printInfo=true);
  declareProperty("FastPedSubtractor", m_fastSubtractor=1);
  declareProperty("ConvergenceLimit", 
                  m_convergenceLimit=FAST_CONVERGENCE_LIMIT);  
  declareProperty("RunType", m_runType=PROCESSING_RUN);
}
//=============================================================================
// Destructor
//=============================================================================
VeloTELL1EmulatorInit::~VeloTELL1EmulatorInit() {}
//=============================================================================
// Initialization
//=============================================================================
StatusCode VeloTELL1EmulatorInit::initialize() {
  StatusCode sc = GaudiAlgorithm::initialize(); // must be executed first
  if ( sc.isFailure() ) return sc;  // error printed already by GaudiAlgorithm

  debug() << "==> Initialize" << endmsg;

  return StatusCode::SUCCESS;
}
//=============================================================================
// Main execution
//=============================================================================
StatusCode VeloTELL1EmulatorInit::execute() {

  debug() << "==> Execute" << endmsg;
  //
  StatusCode sc=getData();
  if(sc) sc=emulatorInit();
  if(sc) writeData();
  //
  return StatusCode::SUCCESS;
}
//=============================================================================
//  Finalize
//=============================================================================
StatusCode VeloTELL1EmulatorInit::finalize() {

  debug() << "==> Finalize" << endmsg;

  return GaudiAlgorithm::finalize();  // must be called after all other actions
}
//=============================================================================
StatusCode VeloTELL1EmulatorInit::getData()
{
  debug()<< " ==> getData() " <<endmsg;
  //
  if(!exist<EvtInfos>(m_evtInfoLoc)){
    // there is no evt bank, so we deal with simulation
    m_mode=SIM_DATA;
    m_initInfo=new VeloProcessInfos();
  }else{
    m_mode=REAL_DATA;
    m_evtInfos=get<EvtInfos>(m_evtInfoLoc);
    m_initInfo=new VeloProcessInfos();
  }
  //
  return ( StatusCode::SUCCESS );
}
//=============================================================================
StatusCode VeloTELL1EmulatorInit::emulatorInit()
{
  debug()<< " ==> emulatorInit() " <<endmsg;
  //
  VeloProcessInfo* initInfo=new VeloProcessInfo();
  if(m_mode==SIM_DATA){
    if(m_simProc.size()){
      initInfo->setSimProcessInfo(m_simProc);
      initInfo->setProcType(m_mode);
      if(!m_convergenceLimit){
        Error("Convergence Limit not set!");
        return ( StatusCode::FAILURE );
      }
      initInfo->setConvLimit(m_convergenceLimit);
      // no test runs for simulation
      initInfo->setRunType(PROCESSING_RUN);
    }else{      
      info()<< " ==> Processing info for SIM_DATA are not set! " <<endmsg;
      return ( StatusCode::FAILURE );
    }
  }else if(m_mode==REAL_DATA){
    EvtInfos::const_iterator evtIt=m_evtInfos->begin();
    EvtInfo* evtInfo=(*evtIt);
    m_realProc=evtInfo->processInfo();
    //
    if(m_printInfo){
      debug()<< " ==> Process Info bits: ";
      for(int i=0; i<8; ++i){
        if((m_realProc>>i)&1){ 
          debug()<< 1;
        }else{
          debug()<< 0;
        }
      }
      debug()<<endmsg;
    }    
    //
    initInfo->setDataProcessInfo(m_realProc);
    initInfo->setProcType(m_mode);
    if(!m_convergenceLimit){
        Error("Convergence Limit not set!", StatusCode::FAILURE);
    }
    initInfo->setConvLimit(m_convergenceLimit);
    initInfo->setRunType(m_runType);
  }else{
    error()<< " ==> Unknow data type! " <<endmsg;
    return ( StatusCode::FAILURE );
  }
  m_initInfo->push_back(initInfo);
  //
  return ( StatusCode::SUCCESS );
}
//=============================================================================
void VeloTELL1EmulatorInit::writeData()
{
  debug()<< " ==> writeData() " <<endmsg;
  //
  if(m_initInfo!=0) put(m_initInfo, m_initInfoLoc);
}
//
