// $Id: VeloTELL1EmulatorInit.h,v 1.6 2007/08/26 16:39:14 szumlat Exp $
#ifndef VELOTELL1EMULATORINIT_H 
#define VELOTELL1EMULATORINIT_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiAlgorithm.h"
#include "Kernel/DecodeConf.h"
#include "Kernel/DecodeCore.h"
#include "Event/EvtInfo.h"
#include "Event/VeloProcessInfo.h"

// from TELL1 event
#include "Event/VeloTELL1Data.h"

/** @class VeloTELL1EmulatorInit VeloTELL1EmulatorInit.h
 *  
 *  The algoritm is meant to facilitate using the TELL1 emulator
 *  inside Vetra. It is essential to run the algoritm as first one
 *  during the TELL1Processing sequence. In the case of real NZS data
 *  algorithm gets the information of processing sequence performed
 *  on TELL1 board from EvtInfo raw bank. In the case of simulated 
 *  data one needs to set up properly processing via options file
 *  Please look at the VetraSimNZS.opts for examples.
 *
 *  @author Tomasz Szumlak
 *  @date   2006-07-10
 */

using namespace VeloTELL1;

class VeloTELL1EmulatorInit : public GaudiAlgorithm {
public: 
  /// Standard constructor
  VeloTELL1EmulatorInit( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~VeloTELL1EmulatorInit( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:

  StatusCode getData();
  void writeData();
  StatusCode emulatorInit();
  void prepareData();

private:

  EvtInfos* m_evtInfos;           /// container for Event Info blocks
  std::string m_evtInfoLoc;       /// TES location of container with EvtInfo
  LHCb::VeloTELL1Datas* m_data;   /// decoded data
  std::string m_dataLoc;          /// TES location of the decoded data
  unsigned int m_mode;            /// type of data to process (sim or real)
  VeloProcessInfos* m_initInfo;   /// init info for emulator
  std::string m_initInfoLoc;      /// TES location of init info. container
  unsigned int m_realProc;        /// processing sequence for real data
  dataVec m_simProc;              /// processing sequence for sim data
  bool m_printInfo;
  unsigned int  m_fastSubtractor; /// set the flag if fast pedSub is in use
  unsigned int m_convergenceLimit;
  unsigned int m_runType;         /// process data or test algorithm

};
#endif // VELOTELL1EMULATORINIT_H
