// $Id: VeloTELL1PedestalSubtractor.h,v 1.4 2007/08/26 16:39:14 szumlat Exp $
#ifndef VELOTELL1PEDESTALSUBTRACTOR_H 
#define VELOTELL1PEDESTALSUBTRACTOR_H 1

// Include files
#include "VeloTELL1Algorithm.h"

/** @class VeloTELL1PedestalSubtractor VeloTELL1PedestalSubtractor.h
 *  
 *
 *  @author Tomasz Szumlak
 *  @date   2007-02-08
 */

class TELL1PedestalProcessEngine;
class TELL1UpdateProcessEngine;

class VeloTELL1PedestalSubtractor : public VeloTELL1Algorithm{
public: 

  enum pedLineValue{
    VARIATION_THRESHOLD=15,
    PEDESTAL=512
  };

  enum ram{
    PED_SUM_SHIFT=10,
    RAM_INIT=512,
    SIGNAL_ADC=590,
    CNT_LIMIT=1024,
    SUM_RAM_INIT=524288 // 512*1024
  };

  //typedef std::vector<signed int> sensorRAM;
  typedef std::vector<int> sensorRAM;
  typedef std::vector<sensorRAM> RAM;
  
  /// Standard constructor
  VeloTELL1PedestalSubtractor(
  const std::string& name, ISvcLocator* pSvcLocator);

  virtual ~VeloTELL1PedestalSubtractor( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:

  StatusCode validationRun();
  StatusCode runPedestalSub();
  StatusCode subtractConstPedestal(const unsigned int tell1);
  StatusCode subtractPedestal(const unsigned int tell1,
                            const unsigned int sensCnt);
  StatusCode prepareMemory();
  StatusCode pedestalSumUpdate(sdataVec& adcs, const unsigned int sensCnt);
  StatusCode saturateSubtractedData(sdataVec& inVector);
  StatusCode writePedestals();
  void extractPedestals();
  void prepareEngine();

private:

  sdataVec m_sensorData;
  sdataVec m_pedestalLine;
  bool m_subtractConstPedestal;
  bool m_subtractPedestal;
  RAM m_pedestalSumMemory;           /// this one stores adc sum
  LHCb::VeloTELL1Datas* m_subPeds;   /// output - subtracted pedestals
  std::string m_subPedsLoc;          /// TES location of the subtracted peds
  unsigned int m_scalingMode;
  TELL1PedestalProcessEngine* m_pedestalEngine;
  LinkPedestalVec m_linkPedestal;
  LinkMaskVec m_linkMask;
  sdataVec m_headers;
  HeaderStrip0Vec m_headerCorrectionValueStrip0;
  TELL1UpdateProcessEngine* m_updateEngine;
  unsigned int m_headerCorrection;
  
};
#endif // VELOTELL1PEDESTALSUBTRACTOR_H
