// $Id: VeloTELL1Reordering.h,v 1.2 2007/08/26 16:39:15 szumlat Exp $
#ifndef VELOTELL1REORDERING_H 
#define VELOTELL1REORDERING_H 1

// Include files
#include "VeloTELL1Algorithm.h"

/** @class VeloTELL1Reordering VeloTELL1Reordering.h
 *    
 * The algorithm implements TELL1 topological channel reordering.
 *
 * Reordering modes: by default there is no reordering performed!
 * If there is problem with data or any else difficulties algorithm
 * just copy the input data. In other case reordering type will be 
 * set dynamically using VeloDet info about sensor type
 *
 * One mandatory variable that should be set up by the user is
 * reordering direction i.e. old position -> new position or
 * new position -> old position (Option file ReorderingDirection)
 *
 *  @author Tomasz Szumlak
 *  @date   2007-02-08
 */

class TELL1ReorderProcessEngine;

class VeloTELL1Reordering : public VeloTELL1Algorithm {

public: 

  enum sectorCorr{
    NO_SECTOR_CORR=0,
    APPLY_SECTOR_CORR=1
  };

  /// Standard constructor
  VeloTELL1Reordering( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~VeloTELL1Reordering( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:

  void prepareEngine();
  void cableErrorCorrection(sdataVec& inVec);
  void trialReorder(sdataVec& inVec);
  void customReorder(sdataVec& inVec);
  StatusCode topologicalChannelReordering();
  StatusCode reorder();
  StatusCode convertChannel(int& oldPos, int& newPos);

private:

  LHCb::VeloTELL1Datas* m_reorderedData;
  int m_funnyfix;
  bool m_printInfo;
  int m_reorderingMode;                  /// mode:1==PHI, 2==R, 0 nothing to do
  TELL1ReorderProcessEngine* m_reorderEngine;
  bool m_customReordering;  // switches on my custom re-ordering algorithm.
};
#endif // VELOTELL1REORDERING_H
