// $Id: VeloChannelReordererMoni.cpp,v 1.4 2007/08/26 19:20:05 szumlat Exp $
// Include files 

// from Gaudi
#include "GaudiKernel/DeclareFactoryEntries.h" 

// local
#include "VeloChannelReordererMoni.h"

// kernel
#include "Kernel/DecodeConf.h"
#include "Kernel/DecodeCore.h"
#include "Kernel/ContPrinter.h"

// boost
#include "boost/lexical_cast.hpp"

//-----------------------------------------------------------------------------
// Implementation file for class : VeloChannelReordererMoni
//
// 2006-10-23 : Tomasz Szumlak
//-----------------------------------------------------------------------------

// Declaration of the Algorithm Factory
DECLARE_ALGORITHM_FACTORY( VeloChannelReordererMoni );

using namespace LHCb;
using namespace VeloTELL1;

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
VeloChannelReordererMoni::VeloChannelReordererMoni( const std::string& name,
                                                    ISvcLocator* pSvcLocator)
  : GaudiTupleAlg ( name , pSvcLocator ),
    m_procInfos ( 0 ),
    m_procInfosLoc ( VeloProcessInfoLocation::Default ),
    m_reorderedLoc ( LHCb::VeloTELL1DataLocation::ReorderedADCs ),
    m_numberOfEvents ( 0 ),
    m_isInitialized ( false )
{
  declareProperty("PlotReorderedData", m_plotReordered=true);
  declareProperty("SubtractorType", m_subType=1);
}
//=============================================================================
// Destructor
//=============================================================================
VeloChannelReordererMoni::~VeloChannelReordererMoni() {}
//=============================================================================
// Initialization
//=============================================================================
StatusCode VeloChannelReordererMoni::initialize() {
  StatusCode sc = GaudiAlgorithm::initialize(); // must be executed first
  if ( sc.isFailure() ) return sc;  // error printed already by GaudiAlgorithm

  debug() << "==> Initialize" << endmsg;
  setHistoTopDir("Vetra/");
  // 
  return StatusCode::SUCCESS;
}

//=============================================================================
// Main execution
//=============================================================================
StatusCode VeloChannelReordererMoni::execute() {

  debug() << "==> Execute" << endmsg;
  //
  m_numberOfEvents++;
  if(!m_isInitialized){
    if(!exist<VeloProcessInfos>(m_procInfosLoc)){
      debug()<< " ==> No init info available! " <<endmsg;
    }else{
      m_procInfos=get<VeloProcessInfos>(m_procInfosLoc);
    }
    m_isInitialized=true;
    VeloProcessInfos::const_iterator procIt=m_procInfos->begin();
    if((*procIt)->convLimit()<=0){
      Error("Convergence Limit not set!", StatusCode::FAILURE);
    }else{
      m_convergenceLimit=(*procIt)->convLimit();
    }
  }
  if(m_numberOfEvents>m_convergenceLimit){
    StatusCode sc=getData();
    if(sc) plotReorderedData();
  }
  //
  return StatusCode::SUCCESS;
}
//=============================================================================
//  Finalize
//=============================================================================
StatusCode VeloChannelReordererMoni::finalize() {

  debug() << "==> Finalize" << endmsg;

  return GaudiAlgorithm::finalize();  // must be called after all other actions
}
//=============================================================================
StatusCode VeloChannelReordererMoni::getData()
{
  debug()<< " ==> getData()" <<endmsg;
  // get ADC samples after channel reordering
  // it does not matter if one deals with real or
  // simulated data 
  if(!exist<LHCb::VeloTELL1Datas>(m_reorderedLoc)){
    error()<< "There is no reordered ADC samples in TES!" 
           << "at location: " << m_reorderedLoc <<endmsg;
    return ( StatusCode::FAILURE );
  }else{
    m_reordered=get<LHCb::VeloTELL1Datas>(m_reorderedLoc);
  }
  //
  return ( StatusCode::SUCCESS );
}
//=============================================================================
StatusCode VeloChannelReordererMoni::plotReorderedData()
{
  debug()<< " plotReorderedData() " <<endmsg;
  //
  if(m_plotReordered){
    VeloTELL1Datas::const_iterator sensIt=m_reordered->begin();
    for( ; sensIt!=m_reordered->end(); ++sensIt){
      sdataVec ALinkADC;
      scdatIt iT;
      unsigned int channel=0;
      int TELL1=(*sensIt)->key();
      VeloTELL1Data* adcData=(*sensIt);
      sdataVec data=adcData->data();
      debug()<< " reordered data in moni: " << data.size() <<endmsg;
      //if(m_numberOfEvents>m_convergenceLimit) contPrinter(data, 5);
      for(int aLink=0; aLink<(ALL_ALINKS); ++aLink){
        ALinkADC=(*adcData)[aLink];
        for(iT=ALinkADC.begin(); iT!=ALinkADC.end(); ++iT){
          plot2D(channel, (*iT), TELL1,
                 "Reordered ADCs data for TELL1 number "+
                 boost::lexical_cast<std::string>(TELL1),
                 0., 2304, -130., 130., 2304, 260);
          ++channel;
        }
      }
    }
  }
  //
  return ( StatusCode::SUCCESS );
}
//
