// $Id: VeloLCMSMoni.h,v 1.2 2007/08/26 19:20:05 szumlat Exp $
#ifndef VELOLCMSMONI_H 
#define VELOLCMSMONI_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiTupleAlg.h"

// from TELL1 event
#include "Event/VeloTELL1Data.h"
#include "Event/VeloProcessInfo.h"
#include "Event/EvtInfo.h"


/** @class VeloLCMSMoni VeloLCMSMoni.h
 *  
 *
 *  @author Tomasz Szumlak
 *  @date   2007-02-09
 */

class VeloLCMSMoni : public GaudiTupleAlg{

public: 
  /// Standard constructor
  VeloLCMSMoni( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~VeloLCMSMoni( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:

  StatusCode getData();
  StatusCode plotLCMSData();

private:

  VeloProcessInfos* m_procInfos;
  std::string m_procInfosLoc;
  LHCb::VeloTELL1Datas* m_LCMSADCs;  /// ADC samples after reordering
  std::string m_LCMSADCsLoc;         /// the container name
  unsigned long m_eventNumber;      /// number of processed events
  bool m_plotLCMS;                  /// plot LCM suppressed data
  bool m_isInitialized;
  unsigned int m_convergenceLimit;
  // hit tree variables
  unsigned int m_hitPosX;
  signed int m_hitADCValue;
  // event tree variables
  unsigned int m_hitId;
  // no of hits on each detector
  unsigned int m_numberOfHitsGla0;
  unsigned int m_numberOfHitsGla1;
  unsigned int m_numberOfHitsGla2;
  unsigned int m_numberOfHitsFre0;
  unsigned int m_numberOfHitsFre1;
  unsigned int m_numberOfHitsFre2;
  // sum of hits on each detector
  signed int m_adcSumGla0;
  signed int m_adcSumGla1;
  signed int m_adcSumGla2;
  signed int m_adcSumFre0;
  signed int m_adcSumFre1;
  signed int m_adcSumFre2;
  // max hit on each detector
  signed int m_adcMaxGla0;
  signed int m_adcMaxGla1;
  signed int m_adcMaxGla2;
  signed int m_adcMaxFre0;
  signed int m_adcMaxFre1;
  signed int m_adcMaxFre2;
  // position of max hit on each detector
  signed int m_posXMaxGla0;
  signed int m_posXMaxGla1;
  signed int m_posXMaxGla2;
  signed int m_posXMaxFre0;
  signed int m_posXMaxFre1;
  signed int m_posXMaxFre2;
  // Thresholds
  signed int m_hitThreshold;
  signed int m_hitThresholdHeader;
  signed int m_hitThresholdGla0;
  signed int m_hitThresholdGla1;
  signed int m_hitThresholdGla2;
  signed int m_hitThresholdFre0;
  signed int m_hitThresholdFre1;
  signed int m_hitThresholdFre2;
  signed int m_headerWidth;

  // Extra event information
  unsigned int m_bunchCounter;
  unsigned long int m_BCntLoop;  /// In this version, this variable both fixes the problems caused by the bunch counter looping back to zero, and also increases with the BATTrig value. So, by sorting the tree by this variable we can get all the "events" from the TELL1 data file in the right order, ready for analysis.
  bool m_doesCounterLoop;

  unsigned int m_BATTrig;
  unsigned int m_orderRaw;


  EvtInfos* m_evtInfos;              /// container for Event Info blocks
  std::string m_evtInfoLoc;          /// location of decoded Event Info

};
#endif // VELOLCMSMONI_H
