// $Id: VeloPedestalSubtractorMoni.h,v 1.3 2007/08/26 19:20:05 szumlat Exp $
#ifndef VELOPEDESTALSUBTRACTORMONI_H 
#define VELOPEDESTALSUBTRACTORMONI_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiTupleAlg.h"

// from TELL1 event
#include "Event/VeloTELL1Data.h"
#include "Event/VeloProcessInfo.h"
#include "Event/EvtInfo.h"

/** @class VeloPedestalSubtractorMoni VeloPedestalSubtractorMoni.h
 *  
 * implementation of the basic monitor for VeloPedestalSubtractor
 * algorithm, one can produce a set of histograms with 
 * ADC samples and sebtracted pedestals
 *
 *  @author Tomasz Szumlak
 *  @date   2006-06-16
 */

class VeloPedestalSubtractorMoni : public GaudiTupleAlg {
public:
  /// Standard constructor
  VeloPedestalSubtractorMoni(
  const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~VeloPedestalSubtractorMoni( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:

  StatusCode getData();
  StatusCode plotSubtractedData();
  StatusCode plotPedestals();
  void setPedFlag();
  void unsetPedFlag();
  bool pedFlag();

private:

  VeloProcessInfos* m_procInfos;
  std::string m_procInfosLoc;
  LHCb::VeloTELL1Datas* m_subADCs;  /// ADC samples after peds subtraction
  std::string m_subADCsLoc;         /// the container name
  LHCb::VeloTELL1Datas* m_peds;     /// container with subtracted pedestals
  std::string m_pedsLoc;            /// the container name
  unsigned long m_numberOfEvt;      /// number of processed events
  bool m_plotSubADCs;               /// peds subtracted data will be plot
  bool m_plotPeds;                  /// if set pedestals will be plot
  bool m_isPed;                     /// check if there are pedestals at TES
  bool m_isInitialized;
  unsigned int m_convergenceLimit;
  // hit tree variables
  unsigned int m_hitPosX;
  signed int m_hitADCValue;
  // event tree variables
  unsigned int m_hitId;
  unsigned int m_numberOfHits;
  signed int m_adcSum;
  signed int m_hitThreshold;
  unsigned int m_bunchCounter;
  EvtInfos* m_evtInfos;              /// container for Event Info blocks
  std::string m_evtInfoLoc;          /// location of decoded Event Info

};
#endif // VELOPEDESTALSUBTRACTORMONI_H
