// $Id: VeloTELL1ADCReader.h,v 1.1 2007/08/26 19:20:05 szumlat Exp $
#ifndef VELOTELL1ADCREADER_H 
#define VELOTELL1ADCREADER_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiTupleAlg.h"
#include "Event/VeloTELL1Data.h"
#include "Event/EvtInfo.h"
#include "Event/VeloODINBank.h"
#include "Event/VeloProcessInfo.h"

/** @class VeloTELL1ADCReader VeloTELL1ADCReader.h
 *  
 *
 *  @author Tomasz Szumlak
 *  @date   2006-05-25
 */

using namespace LHCb;

class VeloTELL1ADCReader : public GaudiTupleAlg {
public: 
  enum consts{
    MAX_ADC=1024
  };

  /// Standard constructor
  VeloTELL1ADCReader( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~VeloTELL1ADCReader( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:

  StatusCode getData();
  StatusCode getAllSensors();
  StatusCode plotAll(VeloTELL1Data*& inData);
  StatusCode plotALinks(VeloTELL1Data*& inData);
  StatusCode dumpEvtInfo(EvtInfo*& anInfo);
  void odinInfo();
  void setADCBankFlag();
  void unsetADCBankFlag();
  void setEvtInfoFlag();
  void unsetEvtInfoFlag();
  bool adcBankFlag();
  bool evtInfoFlag();
  void finalProcess();

private:

  VeloProcessInfos* m_procInfos;
  std::string m_procInfosLoc;
  std::string m_decodedADCLoc;       /// location of decoded ADC in TES
  std::string m_decodedHeaderLoc;    /// location of decoded headers in TES
  std::string m_evtInfoLoc;          /// location of decoded Event Info
  std::string m_odinLoc;             /// location of the decoded ODIN bank
  VeloTELL1Datas* m_decodedADCs;     /// container for ADCs
  VeloTELL1Datas* m_decodedHeaders;  /// container for ADC headers
  EvtInfos* m_evtInfos;              /// container for Event Info blocks
  VeloODINBanks* m_odin;             /// container with ODIN bank
  bool m_isADCBank;                  /// set the flag if ADC data is present
  bool m_isEvtInfo;                  /// flag for the EvtInfo
  bool m_printInfo;                  /// for data dump
  bool m_plotAllData;                /// plot all ADCs vs. all channels
  bool m_plotALinks;                 /// plot ADCs vs. channels for each ALink
  bool m_odinDump;                   /// dump the ODIN bank if the flag is set
  unsigned int m_eventNumber;
  unsigned int m_subType;           /// by default fast subtractor is used (1)
  unsigned int m_convergenceLimit;
  bool m_isInitialized;

  // Setting upper and lower limits on the channels we write to ntuple

  int m_maxChannel;
  int m_minChannel;

  // hit tree variables
  unsigned int m_hitPosX;
  signed int m_hitADCValue;
  // event tree variables
  unsigned int m_hitId;
  unsigned int m_numberOfHits;
  signed int m_adcSum;

};
#endif // VELOTELL1ADCREADER_H
