// $Id: VeloTELL1PedestalsReader.h,v 1.1 2007/08/26 19:20:05 szumlat Exp $
#ifndef VELOTELL1PEDESTALSREADER_H 
#define VELOTELL1PEDESTALSREADER_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiTupleAlg.h"
#include "Event/VeloTELL1Data.h"

/** @class VeloTELL1PedestalsReader VeloTELL1PedestalsReader.h
 *  
 *  Basic monitoring algorithm to plot decoded pedestal values
 *  for each sensor (2048 channels) or for each ALink (32 channels)
 *  The most important options are PlotAllPed and plot AllALinks
 *  by default both are set to false. To dump all pedestal values
 *  (only debugging!) set flag PrintInfo
 *
 *  @author Tomasz Szumlak
 *  @date   2006-05-25
 */

using namespace LHCb;

class VeloTELL1PedestalsReader : public GaudiTupleAlg {
public: 
  /// Standard constructor
  VeloTELL1PedestalsReader( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~VeloTELL1PedestalsReader( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:

  StatusCode getData();
  StatusCode getAllSensors();
  StatusCode plotAllPed(VeloTELL1Data*& inData);
  StatusCode plotALinks(VeloTELL1Data*& inData);
  bool pedBankFlag();
  void setPedBankFlag();
  void unsetPedBankFlag();

private:

  std::string m_decodedPedLoc;
  VeloTELL1Datas* m_decodedPeds;
  bool m_isPedBank;
  bool m_printInfo;
  bool m_plotAllPed;
  bool m_plotALinks;

};
#endif // VELOTELL1PEDESTALSREADER_H
