// $Id: VeloTELL1SimMoni.h,v 1.1.1.1 2006/10/24 14:16:17 szumlat Exp $
#ifndef VELOTELL1SIMMONI_H 
#define VELOTELL1SIMMONI_H 1

// Include files
#include "Event/MCVeloFE.h"

// from Gaudi
#include "GaudiAlg/GaudiTupleAlg.h"
#include "VeloDet/DeVelo.h"

// from TELL1 event
#include "Event/VeloTELL1Data.h"

/** @class VeloTELL1SimMoni VeloTELL1SimMoni.h
 *  
 * the class implements monitor algorithm for non zero suppressed data
 * monitoring is performed in two stages; during the first one fe objects
 * coming directly from silico simulation algorithm are checked
 * during the second stage histograms are booked with raw simulated
 * adc counts and digitized pedestals, for each read-out sensor two
 * histograms are created with adcs counts and pedestals respectively
 *
 *  @author Tomasz Szumlak
 *  @date   2006-05-22
 */

class MCVeloFE;
class IMCVeloFEType;

using namespace LHCb;

class VeloTELL1SimMoni : public GaudiTupleAlg {
public: 
  /// Standard constructor
  VeloTELL1SimMoni( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~VeloTELL1SimMoni( ); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization

protected:

  StatusCode getData();
  StatusCode basicMonitor();
  StatusCode VeloFEMonitor();
  StatusCode VeloTELL1DigiMoni();

private:
  // nzs stands for non-zero suppressed

  bool m_printInfo;               /// set the flag if you want detailed info
  bool m_detailedMonitor;         /// full fe monitoring
  LHCb::MCVeloFEs* m_mcVFEsCont;  /// container with MCVeloFE objects
  bool m_veloFEMoni;              /// flag that indicates presence of fe cont.
  DeVelo* m_veloDet;              /// pointer to DeVelo
  IMCVeloFEType* m_feTypeTool;    /// pointer to type tool
  long m_nMCVeloFE;               /// total number of fe objects
  double m_nMCVeloFE2;            /// total number of fe objects squared
  long m_nMCVeloFEs;              /// number of fe objects from signal
  long m_nMCVeloFEn;              /// from noise
  long m_nMCVeloFEo;              /// from spill-over
  long m_NumberOfEvents;          /// total number of read-in events
  std::string m_adcLocation;      /// TES location of the digitized nzs data
  std::string m_pedLocation;      /// TES location of the digitized pedestals
  VeloTELL1Datas* m_adcs;         /// container with nzs adc objects
  VeloTELL1Datas* m_peds;         /// container with pedestals
  bool m_nzsDataMoni;             /// nzs data minitoring flag
  bool m_plotADCs;                /// plot ADC samples flag
  bool m_plotPeds;                /// plot Pedestals flag

};
#endif // VELOSIMMONI_H
