int savePlot(char plotMsg[])
{
  char plotFile[50];
  char answer;

  cout << plotMsg << ", save plot (y/n/q)";

  //  answer = getchar();
  cin >> answer;

  if ( answer == 'q' ) {
    return 1;
  }

  if ( answer == 'y' ) {
    cout << "File name: ";
    cin >> plotFile;
    c1->Print(plotFile);
  }
  return 0;
}


void phaseScan(char rootFile[]){
  int tell1Id = 30;
  phaseScan(rootFile, tell1Id);  
}


void phaseScan(char rootFile[], int tell1Id){
  cout << "====  ROOT file:  " << rootFile << endl;
  cout << "====  TELL1 ID:   " << tell1Id << endl;
  
  // Open ROOT file and access (sub)directories
  TFile *ff=TFile::Open(rootFile);
  if ( ff == NULL ) {
    cout << " No such file!" << endl;
    return;
  }

  TDirectory *tdVetra = ff->Get("Vetra");
  if ( tdVetra == NULL ) {
    cout << " No Vetra directory " << endl;
    return;
  }

  char dirName[50];
  sprintf(dirName, "TELL1_%d_PhaseScan", tell1Id );
  TDirectory *tdPhaseScan = tdVetra->Get(dirName);
  if ( tdPhaseScan == NULL ) {
    cout << " Directory Vetra/" << dirName << " does not exist" << endl;
    return;
  }

  // Some variables
  int i, j;
  int iCard, cable;
  int iChip, chip, iChipOld;
  int iLink, link, linkOld;
  int iChannel;
  int iPhase;
  int num;
  Double_t sum1, sum2, mean, rms;
  Double_t maxValue;
  char name[50];
  char title[50];
  
  // Create histograms
  TH1F *bestPhase = new TH1F("bestPhase", "Best phase vs link", 64, -0.5, 63.5);
  TH1F *linkInUse = new TH1F("linkInUse", "Links that has noise", 64, -0.5, 63.5);

  TH1F *hCmsNoise[64];
  TH1F *hSignal[64];
  TH1F *hAveNoise[64];
  TH1F *hRatio[64];

  TH1F *hCmsNoiseChip[16];
  TH1F *hSignalChip[16];
  TH1F *hAveNoiseChip[16];
  TH1F *hRatioChip[16];

  TH1F *hCmsNoiseCable[4];
  TH1F *hSignalCable[4];
  TH1F *hAveNoiseCable[4];
  TH1F *hRatioCable[4];
  
  TH1F *evenSignal;
  TH1F *oddSignal;
  TH1F *hSamplingPhaseNoise;

  // Loop over ARx cards, Beetle chips, and links
  for ( iCard = 0; iCard < 4; iCard++ ) {
    for ( iChipOld = 0; iChipOld < 4; iChipOld++ ) {
      for ( iLink = 0; iLink < 4; iLink++ ) {
        iChip = iChipOld;
        //         if ( iChipOld < 2 ) {
        //           iChip = iChipOld+2;
        //         } else {
        //           iChip = iChipOld-2;
        //         }
        linkOld = iCard*16 + iChipOld*4 + iLink;
        link = iCard*16 + iChip*4 + iLink;
        
        // Common mode suppressed mode
        sprintf(name, "hSamplingPhaseNoiseRMSEvenAfterCM_%d", linkOld);
        hSamplingPhaseNoise = (TH1F *)tdPhaseScan->Get(name);

        sprintf(name, "hCmsNoise_%d", link);
        sprintf(title, "Cms noise card %d chip %d link %d", iCard, iChip, iLink);
        hCmsNoise[link] = (TH1F *)hSamplingPhaseNoise->Clone(name);
        hCmsNoise[link]->SetTitle(title);
        if ( hCmsNoise[link]->GetMaximum(1000.0) > 0.1 ) {
          linkInUse->Fill(link);
        }
        
        // Signal (even-odd)
        sprintf(name, "hSamplingPhasePedestalEven_%d", linkOld);
        evenSignal = (TH1F *)tdPhaseScan->Get(name);
        sprintf(name, "hSamplingPhasePedestalOdd_%d", linkOld);
        oddSignal = (TH1F *)tdPhaseScan->Get(name);

        sprintf(name, "hSignal_%d", link);
        sprintf(title, "Signal card %d chip %d link %d", iCard, iChip, iLink);
        hSignal[link] = (TH1F *)evenSignal->Clone(name);
        hSignal[link]->Add(oddSignal, -1.0);
        hSignal[link]->SetTitle(title);
        
        // Average noise
        sprintf(name, "hAveNoise_%d", link);
        sprintf(title, "Average noise card %d chip %d link %d", iCard, iChip, iLink);
        hAveNoise[link] = (TH1F *)hSamplingPhaseNoise->Clone(name);
        hAveNoise[link]->SetTitle(title);

        for ( iPhase = 0; iPhase < 25; iPhase++ ) {

          // Calculate average noise of good channels for different Phase
          num  = 0;
          sum1 = 0;
          sum2 = 0;
          for ( iChannel=0; iChannel < 32; iChannel++ ) {
            if ( iChannel != 0 && iChannel !=1 && iChannel != 3 && iChannel !=4 && iChannel !=5 &&
                 iChannel != 22 && iChannel !=23 && iChannel !=24 ) {
              i = iChannel*25 + iPhase;
              num ++;
              sum1 += hSamplingPhaseNoise->GetBinContent(i+1);
              sum2 += hSamplingPhaseNoise->GetBinContent(i+1)**2;
            }
          }
          mean = sum1/num;
          sum2 = sum2/num - (sum1/num)**2;
          rms  = ( sum2 > 0 ) ? sqrt(sum2) : 0.0;
          for ( iChannel = 0; iChannel < 32; iChannel++ ) {
            i = iChannel*25 + iPhase;
            hAveNoise[link]->SetBinContent(i+1, mean);
            hAveNoise[link]->SetBinError(i+1, rms);
          }
        
          // Signal to noise ratio
          sprintf(name, "hRatio_%d", link);
          sprintf(title, "Signal noise ratio card %d chip %d link %d", iCard, iChip, iLink);
          hRatio[link] =(TH1F *) hSignal[link]->Clone(name);
          hRatio[link]->Divide(hRatio[link], hAveNoise[link], 1.0, 2.0, "");
          hRatio[link]->SetTitle(title);
        }
      }
    }
  }
  
  //Average for each Beetle chip, and each data cable
  for ( iCard = 0; iCard < 4; iCard++ ) {
    for ( iChip = 0; iChip < 4; iChip++ ) {
      chip = iCard*4 + iChip;
      for ( iLink = 0; iLink < 4; iLink++ ) {
        link = iCard*16 + iChip*4 + iLink;
        if ( iLink == 0 ) {
          sprintf(name, "hCmsNoiseChip_%d", chip);
          sprintf(title, "Cms noise card %d chip %d", iCard, iChip);
          hCmsNoiseChip[chip] = (TH1F *)hCmsNoise[link]->Clone(name);
          hCmsNoiseChip[chip]->SetTitle(title);
          
          sprintf(name, "hSignalChip_%d", chip);
          sprintf(title, "Signal card %d chip %d", iCard, iChip);
          hSignalChip[chip] = (TH1F *) hSignal[link]->Clone(name);
          hSignalChip[chip]->SetTitle(title);
          
          sprintf(name, "hAveNoiseChip_%d", chip);
          sprintf(title, "Average noise card %d chip %d", iCard, iChip);
          hAveNoiseChip[chip] = (TH1F *)hAveNoise[link]->Clone(name);
          hAveNoiseChip[chip]->SetTitle(title);
          
          sprintf(name, "hRatioChip_%d", chip);
          sprintf(title, "Signal noise ratio card %d chip %d", iCard, iChip);
          hRatioChip[chip] = (TH1F *)hRatio[link]->Clone(name);
          hRatioChip[chip]->SetTitle(title);
        } else {
          //hCmsNoiseChip[chip]->Add(hCmsNoise[link],1.0);
          hSignalChip[chip]->Add(hSignal[link], 1.0);
          hAveNoiseChip[chip]->Add(hAveNoise[link], 1.0);
          hRatioChip[chip]->Add(hRatio[link], 1.0);
        }
        if ( iLink == 3 ) {
          //hCmsNoiseChip[chip]->Scale(0.25);
          hSignalChip[chip]->Scale(0.25);
          hAveNoiseChip[chip]->Scale(0.25);
           hRatioChip[chip]->Scale(0.25);
        }

        if ( iChip == 0 && iLink == 0 ) {
          sprintf(name, "hCmsNoiseCable_%d", iCard);
          sprintf(title, "Cms noise card/cable %d", iCard);
          hCmsNoiseCable[iCard] = (TH1F *)hCmsNoise[link]->Clone(name);
          hCmsNoiseCable[iCard]->SetTitle(title);
          
          sprintf(name, "hSignalCable_%d", iCard);
          sprintf(title, "Signal card/cable %d", iCard);
          hSignalCable[iCard] = (TH1F *)hSignal[link]->Clone(name);
          hSignalCable[iCard]->SetTitle(title);
           
          sprintf(name, "hAveNoiseCable_%d", iCard);
          sprintf(title, "Average noise card/cable %d", iCard);
          hAveNoiseCable[iCard] = (TH1F *)hAveNoise[link]->Clone(name);
          hAveNoiseCable[iCard]->SetTitle(title);

          sprintf(name, "hRatioCable_%d", iCard);
          sprintf(title, "Signal noise ratio card/cable %d", iCard);
          hRatioCable[iCard] = (TH1F *)hRatio[link]->Clone(name);
          hRatioCable[iCard]->SetTitle(title);
        } else {
          // hCmsNoiseCable[iCard]->Add(hCmsNoise[link], 1.0);
          hSignalCable[iCard]->Add(hSignal[link], 1.0);
          hAveNoiseCable[iCard]->Add(hAveNoise[link], 1.0);
          hRatioCable[iCard]->Add(hRatio[link], 1.0);
        }
        if ( iChip == 3 && iLink == 3 ) {
          //           hCmsNoiseCable[iCard]->Scale(0.0625);
          hSignalCable[iCard]->Scale(0.0625);
          hAveNoiseCable[iCard]->Scale(0.0625);
          hRatioCable[iCard]->Scale(0.0625);
        }
      }
    }
  }
  
  // General setting
  TCanvas *c1 = new TCanvas("c1","",10,10,600,600);
  gStyle->SetHistLineWidth(2);
  gROOT->ForceStyle();

  // CMS noise of channel 0 vs phase
  maxValue = 1.0;
  for ( cable = 0; cable < 4; cable ++) {
    maxValue = hCmsNoiseCable[cable]->GetMaximum(10000.0)>maxValue ? hCmsNoiseCable[cable]->GetMaximum(10000.0) : maxValue;
  }
  maxValue *= 1.1;

  hCmsNoiseCable[0]->SetLineColor(2);
  hCmsNoiseCable[1]->SetLineColor(4);
  hCmsNoiseCable[2]->SetLineColor(6);
  hCmsNoiseCable[3]->SetLineColor(7);
  gStyle->SetHistLineWidth(4);

  c1->DrawFrame(-0.5, 0.0, 1.5, maxValue, "CMS noise of channel 0 vs phase");
  hCmsNoiseCable[0]->Draw("histsame");
  hCmsNoiseCable[1]->Draw("histsame");
  hCmsNoiseCable[2]->Draw("histsame");
  hCmsNoiseCable[3]->Draw("histsame");
  
  c1->Update();
  if ( savePlot("Cms noise of channel 0 vs phase") ) return;

   // Signal phase scan of channel 4
  maxValue = 1.0;
  for ( cable = 0; cable < 4; cable ++) {
    maxValue = hSignalCable[cable]->GetMaximum(10000.0)>maxValue ? hSignalCable[cable]->GetMaximum(10000.0) : maxValue;
  }
  maxValue *= 1.1;

  hSignalCable[0]->SetLineColor(2);
  hSignalCable[1]->SetLineColor(4);
  hSignalCable[2]->SetLineColor(6);
  hSignalCable[3]->SetLineColor(7);
  
  c1->DrawFrame(2.5, -10.0, 5.5, maxValue, "Signal phase scan of channel 4"); 
  hSignalCable[0]->Draw("histsame");
  hSignalCable[1]->Draw("histsame");
  hSignalCable[2]->Draw("histsame");
  hSignalCable[3]->Draw("histsame");

  c1->Update();
  if ( savePlot("Signal phase scan of channel 4") ) return;

  // CMS noise of channel 4 vs phase
  maxValue = 1.0;
  for ( cable = 0; cable < 4; cable ++) {
    maxValue = hCmsNoiseCable[cable]->GetMaximum(10000.0)>maxValue ? hCmsNoiseCable[cable]->GetMaximum(10000.0) : maxValue;
  }
  maxValue *= 1.1;

  hCmsNoiseCable[0]->SetLineColor(2);
  hCmsNoiseCable[1]->SetLineColor(4);
  hCmsNoiseCable[2]->SetLineColor(6);
  hCmsNoiseCable[3]->SetLineColor(7);

  c1->DrawFrame(2.5, 0.0, 5.5, maxValue, "CMS noise of channel 4 vs phase");
  hCmsNoiseCable[0]->Draw("histsame");
  hCmsNoiseCable[1]->Draw("histsame");
  hCmsNoiseCable[2]->Draw("histsame");
  hCmsNoiseCable[3]->Draw("histsame");

  c1->Update();
  if ( savePlot("CMS noise of channel 4 vs phase") ) return;

  // Average noise of link vs phase
  maxValue = 1.0;
  for ( cable = 0; cable < 4; cable ++) {
    maxValue = hAveNoiseCable[cable]->GetMaximum(10000.0)>maxValue ? hAveNoiseCable[cable]->GetMaximum(10000.0) : maxValue;
  }
  maxValue *= 1.1;

  hAveNoiseCable[0]->SetLineColor(2);
  hAveNoiseCable[1]->SetLineColor(4);
  hAveNoiseCable[2]->SetLineColor(6);
  hAveNoiseCable[3]->SetLineColor(7);

  c1->DrawFrame(2.5, 0.0, 5.5, maxValue, "Average noise of link vs phase");
  hAveNoiseCable[0]->Draw("histsame");
  hAveNoiseCable[1]->Draw("histsame");
  hAveNoiseCable[2]->Draw("histsame");
  hAveNoiseCable[3]->Draw("histsame");

  c1->Update();
  if ( savePlot("Average noise of link vs phase") ) return;

  // Signal noise ratio phase scan of channel 4
  maxValue = 1.0;
  for ( cable = 0; cable < 4; cable ++) {
    maxValue = hRatioCable[cable]->GetMaximum(10000.0)>maxValue ? hRatioCable[cable]->GetMaximum(10000.0) : maxValue;
  }
  maxValue *= 1.1;

  hRatioCable[0]->SetLineColor(2);
  hRatioCable[1]->SetLineColor(4);
  hRatioCable[2]->SetLineColor(6);
  hRatioCable[3]->SetLineColor(7);

  c1->DrawFrame(2.5, 0.0, 5.5, maxValue, "Signal noise ratio of channel 4 vs phase");
  hRatioCable[0]->Draw("histsame");
  hRatioCable[1]->Draw("histsame");
  hRatioCable[2]->Draw("histsame");
  hRatioCable[3]->Draw("histsame");

  c1->Update();
  if ( savePlot("Signal noise ratio of channel 4 vs phase") ) return;
}


